%ALS Alterating least squares
%
%  [Z, Y] = als(X,Y,OPT) % pure samples mode
%
%  [Z, Y] = als(X,Z,OPT) % pure features mode
%
% INPUT 
%   X             [M -by- K] training dataset
%   Y             [N -by- K] set of candidate pure samples
%   Z             [M -by- N] set of candidate pure features
%
%   OPT.          options
%    MODE         'samples' (Y), 'features' (Z), or 'auto' mode; in automatic regime the selection of the real mode
%                 will be made using the sizes of input datasets; if both input datasets have the same sizes then 
%    CROP_MODE    'features', 'samples', 'all' (all by default)
%                 automatic selection will fail; by default = 'auto'
%    MAX_ITER     the max number of iterations; by default = 25
%    CRIT         type of stopping criterion: small changes in the solution ('convergence') or
%                 small reconstruction error ('reconstruction'); by default = 'convergence'
%    EPS          value for stopping criterion; by default = 1e-2
%    VERBOSE      force/disable messages: PRPROGRESS(VERBOSE,['%s\n'],MSG) will be run;
%                 by default = []
%
%
% OUTPUT 
%   Y             [N -by- K] set of spectra of pure components
%   Z             [M -by- N] set of concentrations of pure components
%
% DESRIPTION
% An iterative approcach to the least square problem with positive constraints:
% X = Z*Y at X,Y,Z >=0
% 
% Input are a (non-negative) dataset (X) and a set of candidate pure samples (Y) or
% a set of candidate pure features (Z).
%
% SEE ALSO
% SIMPLISMAM, VARIMAXFM, SIMLISMAPS, OPAPS, VARIMAXOM, ALS, OPA

% Copyright: S.Verzakov, s.verzakov@ewi.tudelft.nl 
% Faculty EWI, Delft University of Technology
% P.O. Box 5031, 2600 GA Delft, The Netherlands

% $Id: als.m,v 1.16 2007/03/15 16:10:24 serguei Exp $

function [Z, Y] = als(X,Y,Options)

if nargin < 3
  Options = [];
end

DefOptions.mode      = 'auto'; % 'samples' | 'features' | 'auto'
DefOptions.crop_mode = 'all'; % 'samples' | 'features' | 'all'
DefOptions.max_iter  = 25;
DefOptions.crit      = 'convergence'; % 'convergence' | 'reconstruction'
DefOptions.eps       = 1e-2;
DefOptions.verbose   = [];

Options = updstruct(DefOptions, Options);

%----------------
if isa(X,'dataset') 
  featlab  = X.featlab;
  featsize = X.featsize;
  data_X = X;

  data_X.data = zeros([size(X,1),1]);
  X = +X;
else  
  data_X = [];
end   

if isa(Y,'dataset') 
  data_Y = Y;
  data_Y.data = zeros([size(Y,1),1]);
  Y = +Y;
else  
  data_Y = [];
end  

prprog = exist('prprogress');
if ~prprog & isempty(Options.verbose)
  Options.verbose = 0;
end
%----------------

%if strcmp(Options.mode,'features')
%  X = X';
%  Y = Y';
%end

if size(X,1) > size(Y,1) & size(X,2) == size(Y,2)
  if strcmp(Options.mode ,'features')
    error('Not proper size of pure features dataset');
  end
  Options.mode = 'samples';
elseif size(X,1) == size(Y,1) & size(X,2) > size(Y,2)
  if strcmp(Options.mode ,'samples')
    error('Not proper size of pure samples dataset');
  end
  Options.mode = 'features';
  X = X';
  Y = Y';
elseif size(X,1) == size(Y,1) & size(X,2) == size(Y,2)
  if strcmp(Options.mode ,'auto')
    error('Cannot automatically detect decomposition (sample/feature) mode');
  elseif strcmp(Options.mode ,'features')
    X = X';
    Y = Y';
  end
else  
  error('Not proper size of pure channels/samples dataset');
end

crop_y = 0;
crop_z = 0;
switch Options.crop_mode
case 'all'
  crop_y = 1;
  crop_z = 1;
case 'samples'
  if strcmp(Options.mode,'samples')
    crop_y = 1;
  else % features
    crop_z = 1;
  end
case 'features'
  if strcmp(Options.mode,'samples')
    crop_z = 1;
  else % features
    crop_y = 1;
  end
end

[N, d] = size(X);
[n, d] = size(Y);

SSX = sum(X(:).^2);

if crop_y
  Y = Y.*repmat(1 - 2*(sum(Y,2)<0),[1 d]);
  Y(Y < 0) = 0;
end


for i=1:Options.max_iter
  Z = X / Y;
  ZZ = Z;
  %Z = Z.*repmat(1 - 2*(sum(Z,1)<0),[N 1]);
  if crop_z
    Z(Z < 0) = 0;
  end

  Y_old = Y;
  Y = Z \ X;
  YY = Y;
  %Y = Y.*repmat(1 - 2*(sum(Y,2)<0),[1 d]);
  if crop_y
    Y(Y < 0) = 0;
  end

  %subplot(2,1,1)
  %plot(ZZ)
  %subplot(2,1,2)
  %plot(YY')
  %pause(1)
  
  switch Options.crit
  case {'convergence','conv'}
    err = sum((Y(:)-Y_old(:)).^2)/sum(Y(:).^2);
  case {'reconstruction','rec'}
    err = X - Z*Y;
    err = sum(err(:).^2)/SSX;
  end

  Msg = ['iter ' num2str(i) '; err: ' num2str(err)];
  if prprog
    prprogress(Options.verbose,['%s\n'],Msg);
  elseif Options.verbose
    disp(['als: ' Msg]);
  end  

  if err <= Options.eps
    break;
  end
end


if strcmp(Options.mode,'features')
  ZZ = Y';
  Y = Z';
  Z = ZZ;
  dd = d;
  d = N;
  N = dd;
  clear ZZ dd
end

normalization = sqrt(sum(Y.^2,2));
normalization(normalization==0) = 1;
Y = Y./repmat(normalization,[1 d]);
Z = Z.*repmat(normalization',[N 1]);

if ~isempty(data_X)
  Z = setdat(data_X, Z);
  %Z.featlab = [repmat('conc',[n, 1]), num2str([1:n]','%-1d')];
  Z.featlab = [1:n]';
  Z = remove_spectra_info(Z);
  Z.name = [data_X.name ': extracted channels'];

  Y = dataset(Y);
  %Y.ident    = cellstr([repmat('spectrum',[n, 1]), num2str([1:n]','%-1d')]);
  Y.ident    = [1:n]';
  Y.featlab  = featlab;
  Y.featsize = featsize;
  Y.user     = data_X.user;
  Y.name = [data_X.name ': extracted spectra'];
end

return






