% BASESUBM Baseline subtraction mapping
% 
%  W = BASESUBM(A,IND,CLIPOPT)
% 
% INPUT  
%  A       dataset with a single spectrum
%  IND     wavelength indices of the baseline region
%  CLIPOPT clip negative values to zero? string 'clip' 
%          or noclip' (opt, def: 'clip')
%
% OUTPUT
%  W       baseline subtraction mapping
%
% DESCRIPTION
% Baseline subtraction is a mapping, trained using a single spectrum A.
% From this spectrum, a mean value over the baseline region defined by
% wavelength indices IND is computed. When applied to a dataset with
% spectra, wavelengths, defined by IND, are removed and from remaining
% data, stored mean value is subtracted. By default, possible negative
% values in the result are clipped to zero. In order to switch off the
% clipping, use 'noclip' option.
%
% SEE ALSO
% DATASETS, MAPPINGS

% Copyright: P. Paclik, pavel@ph.tn.tudelft.nl
% Faculty of Applied Physics, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

% $Id: basesubm.m,v 1.3 2005/02/10 15:24:07 pavel Exp $

function w = basesubm(a,ind,clipopt)

	% No arguments given: return untrained mapping.

	if (nargin < 1) | (isempty(a))
		w = mapping(mfilename);
		w = setname(w,'Baseline subtraction mapping');
		return
	end

	if exist('ind','var') & ~isa(ind,'mapping')
		% training
		
		isdataset(a);
		[m,k]=getsize(a);

		if m>1
			prwarning(1,['more than one spctrum supplied,' ...
				     ' using the first data sample']);
			a=a(1,:);
		end
		
		if ~exist('ind','var')
			error(['indices of baseline wavelengths required' ...
			       ' as 2nd parameter']);
		end
	
		% clipping negative values?
		pars.clip=1; % by default, we clip negative values to zero
		if exist('clipopt','var') & ischar(clipopt)
			if strcmp(clipopt,'noclip')
				pars.clip=0;
			else
				error('unknown option');
			end
		end

		x=+a(ind); % baseline region
		
		pars.meanval=mean(x); % mean of values in base-line region
	        % indices of wavelengths we preserve in the data
		pars.output_region=setdiff(1:k,ind); 
		
		w = mapping(mfilename,'trained',pars,getlablist(a),k,k-length(ind));
		w = setname(w,'Baseline subtraction mapping');

	else                      % Second argument is a a mapping: testing.
		w = ind;
		pars = getdata(w);		 	% Unpack the mapping.

		out=+a(:,pars.output_region)-pars.meanval;
		if pars.clip==1
			% clip the possible negative values to zero
			out(find(out<0))=0;
		end
		w = setdat(a,out,w);

		% set the preserved (non-base line) wavelength indices
		% as output feature labels
		w=setfeatlab(w,pars.output_region');

	end

	return



