% BASESUBM Baseline subtraction mapping
% 
%  W = BASESUBM(A,IND,CLIPOPT)
% 
% INPUT  
%  A       dataset 
%  IND     wavelength indices of the baseline region
%  CLIPOPT clip negative values to zero? string 'clip' 
%          or noclip' (opt, def: 'clip')
%
% OUTPUT
%  W       baseline subtraction mapping
%
% DESCRIPTION
% Baseline subtraction is a fixed mapping.
% From this spectrum, a mean value over the baseline region defined by
% wavelength indices IND is computed. When applied to a dataset with
% spectra, a means over wavelengths defined by IND are computed.
% These channels are removed and from remaining
% data, mean values are subtracted. By default, possible negative
% values in the result are clipped to zero. In order to switch off the
% clipping, use 'noclip' option.
%
% SEE ALSO
% DATASETS, MAPPINGS

% Copyright: S. Verzakov, P. Paclik, s.verzakov@tudelft.nl

% $Id: basesubppm.m,v 1.1 2006/05/03 03:47:27 serguei Exp $

function w = basesubppm(a,ind,clipopt)

  if nargin <= 1 
    error('indices are not supplied');
	end

  if nargin < 3 | isempty(clipopt)
    clipopt = 'clip';
  end


  % No dataset given: return fixed mapping.

	if isempty(a)
    if isempty(ind)
      error('indices are not supplied');
    end
    w = mapping(mfilename, 'fixed', {ind, clipopt});
		w = setname(w,'Baseline Subtraction Preprocessing Mapping');
		return
	end

  isdataset(a);
  [m,k]=getsize(a);

	meanval = mean(+a(:,ind),2); % mean of values in base-line region
	ind_rest = setdiff(1:k,ind); 
  featlab  = a.featlab;
  featlab  = featlab(ind_rest,:);
  a.data   = +a(:,ind_rest) - repmat(meanval,[1, length(ind_rest)]);
	

	if strcmp(clipopt,'clip')
		a(logical(+(a<0))) = 0;
	end

	a.featlab = featlab;
  w = a;

	return



