%CCA_TRAIN Canonical Correlation Analysis (training)
%  
%  [cr, XRes, YRes, Options] = cca_train(X,Y)
%  [cr, XRes, YRes, Options] = cca_train(X,Y,Options)
%
% INPUT 
%  X             [N -by- d_X] training input data matrix, N samples, d_X variables
%  Y             [N -by- d_Y] training output data matrix, N samples, d_Y variables
%
%  Options.
%   M_T          T (X scores) normalization: T(:,i)'*T(:,i) = M_T;
%                by default M_T = N-1 (so if columns of X were mean-centered, columns of T are univariated)
%   M_U          U (Y scores) normalization: U(:,i)'*U(:,i) = M_U; 
%                by default M_U = N-1 (so if columns of Y were mean-centered, columns of U are univariated)
%
%   X_centering  do nothing (=[] or 0), do mean centering (=nan), center around some vaue v (=v);
%                by default =[]
%   Y_centering  do nothing (=[] or 0), do mean centering (=nan), center around some vaue v (=v);
%                by default =[]
%   X_scaling    do nothing (=[] or 1), divide each col by std (=nan), divide by some v (=v);
%                by default =[] (scaling does not change T)
%   Y_scaling    do nothing (=[] or 1), divide each col by std (=nan), divide by some v (=v); 
%                by default =[] (scaling does not change U)
%                
% OUTPUT
%  cr    [1- by -n] correlations between canonical components (cr(i)=corr(XRes(:,i),YRes(:,i)))
%        n = min([N d_X d_Y]); n is the theoretical maximum of the number of components 
%        with non-zero cr. In practice the number of such components can be less then n
%        (e.g. due to mean-centering)
%
%  XRes.
%   T    [N -by- n]    scores   (transformed (preprocessed) X)
%   W    [d_X -by- n]  weights  (transformation matrix)
%   P    [d_X -by- n]  loadings (back-transformation matrix or canonical structure)
%
%  YRes.
%   U    [N -by- n]    scores   (transformed (preprocessed) Y)
%   Q    [d_Y -by- n]  weights  (transformation matrix)
%   C    [d_Y -by- n]  loadings (back-transformation matrix or canonical structure)
%
% Options.
%   contains the same fields as in the input, but the values can be changed 
%   (e.g. after mean centering X_centering = mean(X,1))
%
% DESCRIPTION
% Computation of the linear transformations X to T and Y to U such that i-th columns of the
% T and U have maximum possible correlation and at the same time are othogonal to the 
% previous columns in both matrices. Columns of T and U are normalized to be univariated. 
% Because X or/and Y can be not full-ranked, pinv is used in the algorithm.
%
% Relations between matrices (X and Y are assumed to be preprocessed):
% 
% T = X*W 
% P = X'*T*pinv(T'*T)
%
% U = Y*Q
% C = Y'*U*pinv(U'*U)
%
% T'*T = M_T*eye(n) i.e. columns of T are orthogonal and have the length sqrt(M_T); 
% U'*U = M_U*eye(n) i.e. columns of U are orthogonal and have the length sqrt(M_U); 
% if mean-centering was applied to X(Y) and M_T(M_U) = N-1 then the unbaised estimation of 
% variance of each T(U) column is equal to 1. However a few last columns of T(U) can be zeros 
% because X(Y) can be not full-ranked (e.g. due to mean-centering)
% 
%
% SEE ALSO
% CCA_APPLY

% Copyright: S. Verzakov, serguei@ph.tn.tudelft.nl
% Faculty of Applied Sciences, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

% $Id: cca_train.m,v 1.9 2005/02/07 10:03:21 pavel Exp $

function [cr, XRes, YRes, Options] = cca_train(X,Y,Options)

	XRes = [];
	YRes = [];

	if nargin < 3
		Options  = [];
	end

	[N_X, d_X] = size(X);
	[N_Y, d_Y] = size(Y);

	if N_X ~= N_Y
		error('size(X,1) must be equal to size(Y,1)');
	else
		N = N_X;
	end


	DefaultOptions.M_T = N-1;
	DefaultOptions.M_U = N-1;
	DefaultOptions.X_centering = [];
	DefaultOptions.Y_centering = [];
	DefaultOptions.X_scaling = [];
	DefaultOptions.Y_scaling = [];

	Options = updstruct(DefaultOptions, Options);

	[X, Options.X_centering, Options.X_scaling] = prepro(X, Options.X_centering, Options.X_scaling);
	[Y, Options.Y_centering, Options.Y_scaling] = prepro(Y, Options.Y_centering, Options.Y_scaling);

	if N >= d_X
		[Ux,Sx,Vx] = svd(X,0);
	else
		[Vx,Sx,Ux] = svd(X',0);
	end

	if N >= d_Y
		[Uy,Sy,Vy] = svd(Y,0);
	else
		[Vy,Sy,Uy] = svd(Y',0);
	end

	R = Ux'*Uy;

	if size(R,1) >= size(R,2)
		[W,cr,Q] = svd(R,0); 
	else
		[Q,cr,W] = svd(R',0); 
	end

	cr = diag(cr);

	W = sqrt(Options.M_T)*Vx*pinv(Sx)*W;
	Q = sqrt(Options.M_U)*Vy*pinv(Sy)*Q;

	XRes.T = X*W;
	YRes.U = Y*Q;  

	XRes.W = W;
	YRes.Q = Q;

	XRes.P = X'*XRes.T*pinv(XRes.T'*XRes.T);
	YRes.C = Y'*YRes.U*pinv(YRes.U'*YRes.U);

	return;
