%DISSIMM General dissimilarity mapping
%
%   W=DISSIMM(A,TYPE)
%
% INPUT
%   A     Dataset
%   TYPE  string name of a dissimilarity function
%   PAR   optional parameter for the dissimilarity function
%         or a cell array with multiple parameters
%
% OUTPUT
%   W     Dissimilarity mapping
%
% DESCRIPTION
% DISSIMM provides a general dissimilarity mapping for any function with
% two arguments (data and prototypes). While training, prototype objects
% are stored together with a name of a function (TYPE), responsible for the
% computation of dissimilarities. When executed, the specified
% dissimilarity function is invoked again supplying the prototype
% objects as the second argument. If case the function contains
% parameters (e.g. smoothing parameter or the derivative degree), these
% can be specified in a cell array PAR.
%
% EXAMPLES
% constructing an untrained mapping (using dasam.m function):
%  >> w=dissimm('dasam')
% training the dissimilarity mapping:
%  >> w2=a*w
%  Dissimilarity mapping 'dasam', 2 to 100 trained  mapping   --> dissimm
% produce the disismilarity matrix:
%  >> D=b*w2
%  10 by 100 dataset with 2 classes: [5  5]

% $Id: dissimm.m,v 1.5 2005/02/10 15:24:07 pavel Exp $

function w=dissimm(a,type,par)

	if (nargin == 0)
		w = mapping(mfilename,{});
		w = setname(w,['Dissimilarity mapping']);
		return
	end

	% allow simple construction of untrained mapping:
	if nargin==1 & ischar(a), type=a; a=[]; end 
	if isempty(a) & ischar(type)
		if ~exist('par','var')
			w = mapping(mfilename,{type});
		else
			w = mapping(mfilename,{type,par});
		end
		w = setname(w,['Dissimilarity mapping ''' type '''']);
	end

	if isdataset(a)
		% train the dissimilarity mapping
		[m,k,c] = getsize(a);
		if ~exist('par','var'), par=[]; end
		w = mapping(mfilename,'trained',{type,par,a},getlab(a),k,m);
		w = setname(w,'Dissimilarity mapping');
	end
	
	if isa(type,'mapping')
		% execute the mapping
		par=type{2};
		proto=type{3};
		if isempty(par)
			eval(['w=' type{1} '(a,proto);']);
		elseif ~iscell(par)
			eval(['w=' type{1} '(a,proto,par);']);
		else
			str=['w=' type{1} '(a,proto'];
			% parameter is a cell
			for i=1:size(par,2)
				str=[str sprintf(',%f',par{i})];
			end
			str=[str ');'];
			eval(str);
		end
	end

	return
	


