%DSHAPE Spectral distance using shape (Savitsky-Golay smoothing)
%
%   D=DSHAPE(DATA)
%   D=DSHAPE(DATA,PROTO)
%   D=DSHAPE(DATA,PROTO,WINSIZE,DER)
%
% INPUT
%  DATA    testing dataset
%  PROTO   training dataset (representation set)
%  WINSIZE window size for the Saivitsky-Golay filter
%  DER     derivative order (default: 1)
%
% OUTPUT
%  D       dataset with a distance matrix
%   
% DESCRIPTION
% This distance reflects the differences in shape of spectra: It is the L1
% (city-block) distance between smoothed first derivatives of DATA and
% PROTO datasets. Smoothing is performed using a Savitsky-Golay filter with
% window size WSIZE.  By default: WSIZE=11; PROTO=DATA; DER=1;
%
% For the sake of speed, a distfunc MEX file is provided implementing
% the city block distance computation. It is used if present.

% Copyright: Pavel Paclik, pavel@ph.tn.tudelft.nl
% Faculty of Applied Physics, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

% $Id: dshape.m,v 1.5 2005/02/08 10:23:51 pavel Exp $

function d=dshape(data,proto,wsize,der)

if ~exist('proto','var'), proto=data; end
if ~exist('wsize','var'), wsize=11; end
if ~exist('der','var'), der=1; end

if size(data,2)~=size(proto,2)
  error('both datasets must have the same feature sizes');
end
sc=size(data,1); % training sample size
pc=size(proto,1); % test sample size

d=zeros(sc,pc);

labdata=getlab(data);
labproto=getlab(proto);
objsize=data.objsize;
data2=data; data=+data;
proto2=proto; proto=+proto;

ws=smoothm('sg',wsize,1,1);
data=data2*ws;
data=data./(repmat(max(+data,[],2),1,size(data,2))+eps);
proto=proto2*ws;
proto=proto./(repmat(max(+proto,[],2),1,size(proto,2))+eps);

% for computational reasons, we put the lopp over prototypes
% into a mex file dist:
if exist('distfunc','file')
    fprintf(1,'...using the distfunc MEX implementation\n');
    d=distfunc(+data,+proto);
else
    for i=1:pc
	t=repmat(+proto(i,:),sc,1);
	d(:,i)=sum(abs(data-t),2);
    end
end

d=setdata(data2,d,labproto);

return
  
