%DSPEC_SHAPE Spectral distance using shape (Gaussian smoothing)
%
%   D=DSPEC_SHAPE(DATA)
%   D=DSPEC_SHAPE(DATA,PROTO)
%   D=DSPEC_SHAPE(DATA,PROTO,SIGMA,DER,USEDIP)
%
% INPUT
%  DATA    testing dataset
%  PROTO   training dataset (representation set)
%  SIGMA   parameter for the gaussian filter
%  DER     derivative order (default: 1)
%  USEDIP  use DIPImage for smoothed derivatives (default: 1 i.e. run
%          convolution in Matlab)
%
% OUTPUT
%  D       dataset with a distance matrix
%   
% DESCRIPTION
% This distance reflects the differences in shape of spectra:
% It is the L1 (city-block) distance between smoothed first derivatives of
% DATA and PROTO datasets. Smoothing is performed by convolving with the
% gaussian filter with the parameter SIGMA. By default: SIGMA==1;
% PROTO==DATA.

% Copyright: Pavel Paclik, pavel@ph.tn.tudelft.nl
% Faculty of Applied Physics, Delft University of Technology
% P.O. Spectral distance using shape (Gaussian smoothing)

% $Id: dspec_shape.m,v 1.13 2005/02/10 15:24:07 pavel Exp $

function d=dspec_shape(data,proto,sigma,der,usedip)

%fprintf(' dspec_shape:');

if ~exist('proto','var'), proto=data; end
if ~exist('sigma','var'), sigma=1; end
if ~exist('der','var'), der=1; end
if ~exist('usedip','var'), usedip=1; end

if size(data,2)~=size(proto,2)
  error('both datasets must have the same feature sizes');
end
sc=size(data,1); % training sample size
pc=size(proto,1); % test sample size

d=zeros(sc,pc);

labdata=getlab(data);
labproto=getlab(proto);
objsize=data.objsize;
data2=data; data=+data;
proto2=proto; proto=+proto;

if usedip==1
    fprintf(1,'using DIPImage\n');
    data=double(dip_gauss(dip_image(+data),[1 0],[sigma sigma],[der 0]));
    proto=double(dip_gauss(dip_image(+proto),[1 0],[sigma sigma],[der 0]));
else
    % use standard Matlab convolution
    fprintf(1,'using derfilter\n');
    data=derfilter(+data,sigma);
    proto=derfilter(+proto,sigma);    
end

data=data./repmat(max(data,[],2),1,size(data,2));
proto=proto./repmat(max(proto,[],2),1,size(proto,2));

% for computational reasons, we put the lopp over prototypes
% into a mex file dist:
if exist('distfunc')==3
    d=distfunc(data,proto);
else
    for i=1:pc
	t=repmat(proto(i,:),sc,1);
	d(:,i)=sum(abs(data-t),2);
    end
end

if prversion>3.2
  d=dataset(d,labdata,'featlab',labproto);
else
%  d=dataset(d,labdata,labproto);
  d=setdata(data2,d,labproto);
end

% the distance dataset will inherit the objsize from the input dataset
d=setobjsize(d,objsize);

return
  
