% EdgeDetection Detecting edge in hyperspectral image
%
%    A = EDGEDETECTION(DATASET, ALG)
%
% INPUT
%   DATASET name of dataset
%   ALG   edge detection algorithm ('Zenzo' or 'Cluster')
%
% OUTPUT
%   A      dataset

% DESCRIPTION
% Di Zenzor is a gradient based edge detection method, which extended the
% concept of gradient criteria for grayscale image to multi-demensional image.
% Algorithm.
% Step 1. Extract each band from hyperspectral image
% Step 2. Calculate gradient value for each pixel
% Step 3. Determine threshold based on gradient's histogram
% Step 4. Create final edge image of hyperspectral

% Copyright: Dinh Viet Cuong, d.v.cuong@tudelft.nl
% ICT group, Elect. Eng., Maths and Comp. Sc, 
% Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

% $Id: edgedetection.m,v 1.1 2009/01/16 14:55:35 dcuong Exp $

function edge = edgedetection(a,m)

	if nargin == 0
		error('error!, input dataset name');
	elseif ~isa(a,'dataset')
		error('error!, input dataset name');	
	else
		switch lower(m)
		  case {'zenzo','ze'}
	
			% Get width, height, and nChannels of hyperspectral image
			[imheight, imwidth] = getobjsize(a);
			
		  
		  	% Calculate gradient and edgethreshold values
			[gradient, edgethresh] = calgradient(a);
			
			edge = zeros(imheight, imwidth);
			
			%edgethreshold = mean(mean(gradient1))
			
			x = find(gradient < edgethresh);
			edge(x) = 0;
			x = find(gradient > edgethresh);
			edge(x) = 1;
		
		  case {'cluster','cl'}
		  	
			% remove the label
			data = +a;
			
			% create a subset of a
		  	b = gendat(data,1500);
			
                        % Use it for finding 5 clusters, for example
			[lab,w] = emclust(b,nmc,4);
			
                  	% Classify the entire image and show it
			c = a*w;
			
			e = spatm(c)*maxc;
			
			%figure; classim(e);
			
			% check the confidence belonging to a cluster of a pixels
			[conf,J] = max(e*classc,[],2);
			
			labels = reshape(conf,getobjsize(a));
			
			% Convert data to image
			im = data2im(a);
			
			% Get width, height, and nChannels of hyperspectral image
			[imheight, imwidth, nim] = size(im);
			
			edge = zeros(imheight, imwidth);

			% if conf of a pixel is smaller than 0.55, it is considered as edge candidate
			thresh = 0.55;
			
			x = find(labels <= thresh);
			edge(x) = 1;
			x = find(labels > thresh);
			edge(x) = 0;
			%show(edge);
			
			% Calculate gradient value for these pixels;
			[gradient, edgethresh] = calgradient(a);
			
			% gradient threshold is 0.5
			edgethresh = edgethresh /2	
			% refine the result
			%x = find(gradient <= edgethresh);
			%edge(x) = 0;
			
		  case {'gradientcluster','gc'}

		        % Stage 1. Calculate gradient value for each band
		  	
			% Step 1. Smooth hyperspectral image
			sigma = 1; % assign default sigma value;
			smoothimage = datgauss(a, sigma);
			
			% Step 2. Convert dataset to image
			im = data2im(smoothimage);
			
			% Get width, height, and nChannels of hyperspectral image
			[imheight, imwidth, nim] = size(im);
			
			dy = zeros(imheight, imwidth, nim); 
			dx = zeros(imheight, imwidth, nim);

			% Step 3. Calculate the first derivative of the image
			for i = 1: nim
				dy(:,:,i) = [diff(im(:,:,i)); zeros(1, imwidth)]; % add the border
				dx(:,:,i) = [diff(im(:,:,i)')' zeros(imheight,1)]; % add the border
				gradient(:,:,i) = dx(:,:,i).^2 + dy(:,:,i).^2;
				
				%gradient(:,:,i) = sqrt(gradient(:,:,i));
				% Normalize the gradient
				%gradient(:,:,i) = gradient(:,:,i) ./ sum(sum(gradient(:,:,i)));
			end
			
			% Step 2. Cluster in feature bases
			
			% Create dataset
			d = im2feat(gradient);
			d = dataset(+d, [], 'objsize', [imheight imwidth]);
			
			%struct(d)
			
		  	b = gendat(d,1500);
			
                        % Use it for finding 6 clusters
			[lab,w] = emclust(b,nmc,6);
			
			% Classify the entire image and show it
			c = d*w;
			
			e = spatm(c)*maxc;
			%e = spatm(c)*prodc;
			
			labs = e*labeld;
			setlabels(d,labs);
			
			figure; scatterd(d);
			%show(e);
			%sprintf('structure of c:')
			%struct(c)
			
			%sprintf('data of c:')
			%double(c*classc)
					
			%size(c)
			%labels = c*labeld
			%size(labels)			
			
			%e = spatm(c)*maxc;
			% replace this function
			%figure; classim(c);

			% Merge non-background clusters into one cluster
			
			%lab = e*labeld;
			%seg = segment_comb(setlabels(d,lab),5);
			%show(seg);
									
			[mx,J] = max(double(e*classc),[],2);
			
			
			total = zeros(1, size(c,2));
			
			t = unique(J(:));
			count = hist(J(:),t);
			
			[maxv pos] = max(count);
			
			x = find(J == pos);
			labels(x) = 1;
			
			x = find(J ~= pos);
			labels(x) = 2;
		
			labels = reshape(labels,getobjsize(c));
			edge = labels;
			clear labels;
 		 end
	end
end		
	
function [gradient, edgethresh] = calgradient(a)

%input: a: input dataset;
%output: gradient: gradient value
%        edge threshold value

	% Step 1. Smooth hyperspectral image
	sigma = 1; % assign default sigma value;
	smoothimage = datgauss(a, sigma);
			
	% Step 2. Convert dataset to image
	im = data2im(smoothimage);
			
	% Get width, height, and nChannels of hyperspectral image
	[imheight, imwidth, nim] = size(im);
			
	gradient_x_x = zeros(imheight, imwidth); 
	gradient_x_y = zeros(imheight, imwidth); 
	gradient_y_y = zeros(imheight, imwidth);
			
	d1 = zeros(imheight, imwidth); 
	d2 = zeros(imheight, imwidth);
			
	gradient1 = zeros(imheight, imwidth); 	
	gradient2 = zeros(imheight, imwidth); 
			
	% Step 3. Calculate the first derivative of the image
	for i = 1: nim
		dy(:,:,i) = [diff(im(:,:,i)); zeros(1, imwidth)]; % add the border
		dx(:,:,i) = [diff(im(:,:,i)')' zeros(imheight,1)]; % add the border
				
		% Step 4. Calculate the gradient strength of each image
		gradient_x_x = gradient_x_x + dx(:,:,i).^2;
		gradient_y_y = gradient_y_y + dy(:,:,i).^2;
		gradient_x_y = gradient_x_y + dx(:,:,i) .* dy(:,:,i);
	end
			
	% Step 5. Calculate the global gradient direction of every pixel in hyperspectral image
	m = gradient_x_x - gradient_y_y;
			
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	% There are 2 possible result for atan function, 
	% the first one is smaller than pi/2
	% the other is larger than pi/1
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
			
	m(m ==0) = 1e-10;
	d1 = 0.5 * atan2((2*gradient_x_y) , m);
			
	y = d1 < 0;
	d1 = d1 + pi*y;
			
	y = d1 > pi/2;
	d1 = d1 - pi*y;
	d2 = d1 + pi/2;
			
	% Step 6. Calculate the global gradient strength of every pixel in hyperspectral. 
	% One gradient is maximum value, and the other is minimum value.
	% Select the larger one.
			
	gradient = sqrt(0.5*( (gradient_x_x + gradient_y_y) +...
	            (gradient_x_x - gradient_y_y) .* cos(2*d1) + 2 *...
	            gradient_x_y .* sin(2*d1)));
			
	gradient2 = sqrt(0.5*( (gradient_x_x + gradient_y_y) +...
	            (gradient_x_x - gradient_y_y) .* cos(2*d2) + 2 *...
	             gradient_x_y .* sin(2*d2)));
			
	y = find(gradient < gradient2);
	gradient(y)=gradient2(y);
						
	% Normalize the global gradient
	gradient = gradient ./ sum(sum(gradient));
			
	% Step 7. Determine threshold for edge detection based on histogram
	g = reshape(gradient, 1, imwidth*imheight);
			
	[his,val] = hist(g);
			
	cout = 0;t = 1;
	
	% for zenzor method
	%threshold = 0.6 * imwidth * imheight;
	
	% for clustering method
	threshold = 0.75 * imwidth * imheight;
			
	while cout < threshold  
		cout = cout + his(1,t);
	 	t = t+1;
       end
			
	edgethresh = val(1,t);

end
  	
