%FEATSELF_SIMPLE Simple forward feature selection
% 
% 	[W,R] = FEATSELF(A,CRIT,K,T)
% 
% Forward selection of K features using the dataset A. CRIT sets the 
% criterion used by the feature evaluation routine FEATEVAL. If the 
% data set T is given, it is used as test set for FEATEVAL. For K=0 
% the optimal feature set (maximum value of FEATEVAL) is returned. 
% The result W can be used for selecting features by B*W.  Defaults: 
% crit='NN', K=0. In R the search is step by step reported:
% 
% 	R(:,1) : number of features
% 	R(:,2) : criterion value
% 	R(:,3) : added / deleted feature
% 
% See also MAPPINGS, DATASETS, FEATEVAL, FEATSELO, FEATSELB, FEATSELI,
% FEATSELP, FEATSELM

% Copyright: R.P.W. Duin, duin@ph.tn.tudelft.nl
% Faculty of Applied Sciences, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

function [W,R,info] = featself_simple(A,crit,ksel,T,initset)
if nargin < 2, crit = 'NN'; end
if nargin < 3, ksel = 0; end
if nargin < 4, T = []; end

if nargin == 0 | isempty(A)
	W = mapping('featself',{crit,ksel,T});
	W = setname(W,'Forward Feature Selection');
    return
end

[m,k,c] = getsize(A);
featlist = getfeatlab(A);

if ~isempty(T)
	if k ~= size(T,2)
		error('Data sizes do not match')
 	end
end

Cmax = 0;
if ~exist('initset','var')
    I = [1:k];
    J = [];
else
    % we start from a given subset
    I = setdiff(1:k,initset);
    J = initset;
end
R = [];
Iopt = J;
iter=1;
crit_eval=0; % number of crit evaluation
thesameres=0;
while length(J) < k
	C = zeros(1,length(I));
	fprintf(1,'[%d: (%d) ',iter,length(I));	
	for j = 1:length(I)
		L = [J,I(j)];
		if isempty(T)
			C(j) = feateval(A(:,L),crit);
		else
			C(j) = feateval(A(:,L),crit,T(:,L));
		end
		if C(j) > Cmax
			Cmax = C(j);
			Iopt = L;
		end
%		fprintf(1,'.');
	end
	crit_eval=crit_eval+length(I);
	
	[mx,j] = max(C);
	J = [J,I(j)];
  	I(j) = [];
	
	% is the criterion is a classification performance
	% and we reached full separability, stop
	if isa(crit,'mapping') & Cmax==1
	   break
	end
	r = [length(J),mx,J(end)];
	disp(r)
	R = [R; r];
	% increase counter if the same result happens again
	if (size(R,1)>1)
		if R(end-1,2)==R(end,2)
			thesameres=thesameres+1; 
			fprintf(1,'T%d ',thesameres);
		else
			thesameres=0; 			
		end
	end
	% stopping criterion:
	% stop, when reaching requested feature count or perfect
        % separation or if we're above 95% accuracy and the same value repeats 10
        % times and the improvement is smaller then 0.5% (simple, hee? :-)
	if length(J) == ksel | r(2)==1.0 | ...
		      (size(R,1)>1 & (R(end,2)/R(end-1,2))<1.005 & ...
		       ( R(end,2)>=0.95 | thesameres>10 )  )
	   % increase less then a percent?
	   info.crit_eval=crit_eval;	   
	   W = cmapm(k,J);
	   W = setlabels(W,featlist(J,:));
	   W = setname(W,'Forward Feature Selection');
	   return
	end
	iter=iter+1;	
end
info.crit_eval=crit_eval;
W = cmapm(k,Iopt);
W = setlabels(W,featlist(Iopt,:));
W = setname(W,'Forward Feature Selection');
return
