%GENFEATSEL Genetic feature selection algorithm
%
%       [W,P,E]=GENFEATSEL(DATA,N,GENC,WIN,T,PM)
%
% INPUT
%    DATA   original dataset
%    N      population size (default N=500)
%    GENC   generation count (default genc=30)
%    WIN    untrained classifier to be used estimate error
%           (opt, def: fisherc)
%    T      feasibility threshold on error (opt,def:0.05)
%    PM     mutation rate (opt,def: Pm=0.05)
%    DIM    number of features switched on initially
%           (opt,def: dim=[2 size(data,2)])
%
% OUTPUT
%    W     feature selection mapping (the best subset found)
%    P     the final population (uint8 matrix, each row is a
%          binary-encoded feature subset)
%    E     error rate for each population individuum
%
% DESCRIPTION
% Genetic algorithm-based feature selection.
%
% REFERENCE 
% Siedlecki, Sklansky, A note on genetic algorithms for
% large-scale feature selection, Pattern Recognition Letters, vol.10,
% pp.335-347, 1989

% Copyright: P. Paclik, pavel@ph.tn.tudelft.nl
% Faculty of Applied Physics, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

% $Id: genfeatsel.m,v 1.6 2005/02/10 15:24:07 pavel Exp $

function [w,P,e]=genfeatsel(data,N,genc,win,t,Pm,dim)

	if ~exist('N','var'), N=500; end
	if ~exist('genc','var'), genc=30; end	
	if ~exist('win','var'), win=fisherc; end			
	if ~exist('t','var'), t=0.05; end		
	if ~exist('Pm','var'), Pm=0.05; end	
	if ~exist('dim','var'), dim=[2 size(data,2)]; end
	
	m=0.01; % error margin
	const=1/(exp(1)-1);

	f=zeros(N,1); % fitness function
	newf=f;
	
	epsilon=1.01;

	fc=size(data,2); % feature count
	
	P=create_init_pop(N,fc,dim);
	M=P; % mating set

	fc2=floor(fc/2); % index of a band half-way 
	N2=floor(N/2); % index of a population sample half-way
	
	% evaluate initial population
	l=sum(P,2); % number of features per individual
	e=get_err(data,P,win); % population errors
	J=l+const*(exp((e-t)./m)-1); % criterion
	% compute the fitness function
	for i=1:N
		f(i)=epsilon*max(J-J(i));
	end
	[temp,ind]=sort(-f);
		
	if all(f==0)
		error('zero criterion');
	end
	
	for g=1:genc
		fprintf(1,'[%d',g);

		% choose the mating set
		
		cumf=cumsum(f);
		x=floor(rand(1,N)*N);

		for i=1:N
			temp=find(cumf>=x(i));
			M(i,:)=P(temp(1),:);
		end
		
		fprintf(1,'.');
		
		% crossover
		
		temp=randperm(fc); % crossover features are selected randomly 
		
		M=[M(randperm(N),temp(1:fc2)), M(randperm(N),temp(fc2+1:fc))];

		% evaluate crit on new members
		newe=get_err(data,M,win);
		newJ=sum(M,2)+const*(exp((newe-t)./m)-1); % criterion
		% compute the fitness function
		for i=1:N
			newf(i)=epsilon*max(newJ-newJ(i));
		end

		% mutate the new offspring
		temp=find(rand(N,fc)<=Pm);
		M(temp)=not(M(temp));

		% get the best from both sets
		[temp,newind]=sort(-[f;newf]);
		newind=newind(1:N); % indices in the combined population
		temp=[P; M]; % combined population
		P=temp(newind,:);

		% update the population		
		temp=[f;newf]; f=temp(newind);
		temp=[e;newe]; e=temp(newind);
		temp=[J;newJ]; J=temp(newind);		
		l=sum(P,2);

		fprintf(1,':]');
	end
	
	[temp,ind]=sort(J); % sort according to the criterion
	P=P(ind,:); % update the order
	e=e(ind);
	ind=find(P(1,:)==1); % feature indices

	w=cmapm(fc,ind);
	
	return
	
	
% N - number of individuals to create
% d - number of dimensions (features) we search in
% dim - how many dimensions should be switched on
%       Dim may be a vector specifying lower and upper bound)
function P=create_init_pop(N,d,dim)

	% population matrix
	P=repmat(uint8(0),N,d);

	[ignore,ind]=sort(rand(N,d),2);
	clear ignore
	
	% feature count per individual:
	if length(dim)==1
		% constant number of switched on-dimensions
		fc=dim*ones(N,1);
	else
		fc=dim(1)+floor(rand(N,1)*dim(2));
	end
	
	for i=1:N
		P(i,ind(i,1:fc(i)))=1;
	end
	
	return
	
% measure the error for each feature setup (chromosome) in P
% on data using user-defined mapping w
function e=get_err(data,P,win)
	
	e=zeros(size(P,1),1);
	for i=1:size(P,1)
		ind=find(P(i,:)>0);
                w=data(:,ind)*win;
		e(i)=data(:,ind)*w*testc;
	end
	
	return
	
