% GET_PERCENTILE computes percentiles of a data sample
%
%   PERC=GET_PERCENTILE(DATA,P,DIM)
%
% INPUT
%   DATA   data sample (double array or dataset)
%   P      percentiles (may be a vector)
%   DIM    defines along which direction percentile should be computed
%          optional; by default the first non-singleton is used (if data is not a scalar)
%
% OUTPUT
%   PERC   scalar, vector, or matrix of doubles with percentiles
%
% DESCRIPTION
% For 0 < P < 1, this routine returns 100*P percentiles of DATA.
% Multidimensional arrays and PRTools datasets (will be converted into double) are supported.
%
% Percentiles will be calculated along the direction DIM. By default DIM is the first
% non-singleton direction. If DATA is K-by-...-by-SIZE(DATA,DIM)-by-...-by-M matrix and P is 
% a vector of length L then PERC has dimensions M-by-...-by-L-by-...-by-K.
%
% perc = get_percentile((1:5),[0.5, 0.75]);
% returns perc == [3, 4];
%
% perc = get_percentile((1:5)',[0.5, 0.75]);
% returns perc == [3; 4];
%
% perc = get_percentile([(1:5)', (11:15)'],[0.5, 0.75]);
% returns perc == [[3; 4], [13; 14]];
%
% z = cat(3,[1 1; 1 11], [2 2; 2 12], [3 3; 3 13], [4 4; 4 14], [5 5; 5 15]); 
% perc = get_percentile(z,[0.5, 0.75], 3);
% returns perc == cat(3, [3 3; 3 13], [4 4; 4 14]);
%
% to ensure that percentiles are always calculated for featues 
% over the objects of dataset use 
% perc = get_percentile(data,p,1);

% $Id: get_percentile.m,v 1.2 2005/10/19 11:33:37 serguei Exp $

function perc = get_percentile(data,p,dim)
  if nargin < 3 
    dim = [];   
  end    

  if any(p>=1.0 | p<=0)
		error('0 < p < 1 is required');
	end
	
  if isempty(data)
    error('can''t compute percentiles on empty data');
  end

  if isdataset(data)
    data = +data;
    if size(data,1) == 1 & size(data,2) ~= 1 & ~isequal(dim,1)
      prwarning(1,'only one object in dataset; percentile(s) will be computed over features');  
    end
  end  
  
  if isempty(dim)
    sz  = size(data);
    dim = find(sz > 1);
    if isempty(dim)
      dim = 1;
      %error('can''t compute percentile on scalar');  
    else
     dim = dim(1);
    end
  end
  
  %if size(data,1)==1, data=data'; end
	perm  = [dim, (1:(dim-1)), ((dim+1):length(size(data)))];
  data  = permute(data,perm); 
	
  p = p(:);
  data = sort(data,1);
  t = size(data,1)*p;

	p_size = size(data);
  p_size(1) = length(p);
  perc =zeros(p_size);
  
	for i=1:length(p)
		if t(i)==floor(t(i))
			% t is integer
			perc(i,:)=mean(data([t(i) t(i)+1],:));
		else
			perc(i,:)=data(floor(t(i))+1,:);
		end
	end
		
	% compatibility
  %if length(size(data)) == 2 & size(data,2) == 1, return, end 
  
  perc = ipermute(perc,perm);
  
  
  
  
  return
