%GLDBM Generalised Local Discriminant Bases GLDB (bottom-up, Fisher crit)
% 
%  W = GLDBM(A)
% 
% INPUT  
%  A    Training dataset
%
% OUTPUT
%  W    best bases mapping
%
% DESCRIPTION 
% Generalised Best Bases mapping is a feature extraction
% technique for spectral datasets with two classes. Groups of adjacent
% features (wavelengths) are identified in the input dataset such that
% criterion combining Fisher separability and inter-band correlation is
% maximized in each group. Output dataset is created by
% projecting wavelengths in each group to the 1D Fisher subspace.  The
% procedure is defined for two class problems, if a multi-class dataset is
% supplied, a cell array of mappings will be computed.  A multi-class
% classifier, based on this routine, is implemented as PREXP algorithm
% ALG_GLDBM
%
% REFERENCE 
% Kumar,Ghosh,Crawford: Best-Bases Feature Extraction Algorithms for
% Classification of Hyperspectral Data, IEEE Trans.on Geoscience and
% Remote Sensing, bol.39, no.7, July 2001
%
% SEE ALSO
% DATASETS, MAPPINGS, ALG_GLDBM

% Copyright: P. Paclik, pavel@ph.tn.tudelft.nl
% Faculty of Applied Physics, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

% $Id: gldbm.m,v 1.10 2005/02/10 15:24:07 pavel Exp $

function [w,crit] = gldbm(a,arg2)

	% No arguments given: return untrained mapping.

	if (nargin < 1) | (isempty(a))
		w = mapping(mfilename);
		w = setname(w,'Best Bases mapping');
		return
	end

	if (~exist('arg2','var'))     % Second argument is not a mapping: training.

		isdataset(a);           % Assert that A is a dataset.

		[m,k,c] = getsize(a); M = classsizes(a);
		
		if c>2
			% for multi-class problem, create all pair-wise extractors
			w={};
			iter=[];
			for i=1:c-1
				for j=i+1:c
					fprintf(1,'(%d-%d',i,j);
					t=seldat(a,[i j]);
					eval(['w{i,j}=' mfilename '(t);']);
					fprintf(1,') ');
				end
			end
			return
		end

		% counter with ten steps
		t=floor(k/10); % usually, only 80% bands is processed
		counts=t*ones(1,10);
		counts(1:mod(m,10))=counts(1:mod(m,10))+1;
		counts=cumsum(counts);
		
		Q=corrcoef(+a);
		[U,G]=meancov(a);

		m=0;              % level
		Nm=size(a,2);
		l=1:size(a,2); % lower bounds of group-bands
		u=1:size(a,2); % upper bounds of group-bands

		% compute crit for each single band group:
		I=crit_eval(l,u,Q,+U(1,:),+U(2,:),G(:,:,1),G(:,:,2));

		while 1
			% make possible merges:
			pl=l(1:Nm-1);
			pu=u(2:Nm);
			
			% eval criterium
			crit=crit_eval(pl,pu,Q,+U(1,:),+U(2,:),G(:,:,1),G(:,:,2));
			[maxval,ind]=max(crit);
			while length(crit)>0
				% if the merge candidate with the
                                % highest crit.value yields smaller
                                % criterium then maximum of the original
                                % groups, choose 2nd best (3rd,... until
                                % the end
				if maxval<max(I([ind ind+1]))
					crit(ind)=[];
					[maxval,ind]=max(crit);
				else
					break
				end
			end
			if length(crit)==0
				% no merge candidate left, return what
                                % we have at the moment.
				[crit,clf]=crit_eval(l,u,Q,+U(1,:),+U(2,:),G(:,:,1),G(:,:,2));

				% parameters
				pars.clf=clf; pars.l=l; pars.u=u; pars.info_iter=m;
				w = mapping(mfilename,'trained',pars,(1:length(l))',k,length(pars.l));
				w = setname(w,'Best Bases mapping');
				
				return
			end
			
			% update crit vector
			I(ind)=maxval;
			I(ind+1)=[];

			u(ind)=u(ind+1);
			u(ind+1)=[];
			l(ind+1)=[];

			m=m+1;
			Nm=Nm-1;
			try
				if m==counts(1)
					counts(1)=[];
					fprintf('.');
				end
			catch
				fprintf(1,'XX');
% $$$ 				whos
% $$$ 				m
% $$$ 				counts
% $$$ 				pause
			end
				

			if Nm==1
				% all the features were lumped together
				[crit,clf]=crit_eval(l,u,Q,+U(1,:),+U(2,:),G(:,:,1),G(:,:,2));
				% parameters
				pars.clf=clf; pars.l=l; pars.u=u; pars.info_iter=m;
				w = mapping(mfilename,'trained',pars,(1:length(l))',k,length(pars.l));
				w = setname(w,'Best Bases mapping');
				
				return; 
			end
		end

	else                      % Second argument is a a mapping: testing.

		w = arg2;			
		pars = getdata(w); % Unpack the mapping.
		[m,k] = getsize(a); 

		% create an output dataset
		out=[];
		for i=1:length(pars.clf)
			% get the subset of bands...
			t=+a(:,pars.l(i):pars.u(i));
			% ... and project it to 1D Fisher subspace
			out=[out t*pars.clf(i).w];
		end
	
		w = setdat(a,out);
		w=setfeatlab(w,(1:size(w,2))');
	end

return



function [I,clf]=crit_eval(l,u,Q,mua,mub,sigmaa,sigmab)

	I=zeros(1,length(l));

	clf=[];
	for i=1:length(l)
		
		C=min(min( Q( l(i):u(i), l(i):u(i) ) ));
		
		W=0.5*( sigmaa( l(i):u(i), l(i):u(i) )+sigmab( l(i):u(i), l(i):u(i) ) );
		t=( mua( l(i):u(i) ) - mub( l(i):u(i) ) )';
		B=t*t';
		
		w=pinv(W)*t;
		D=(w'*B*w)/(w'*W*w);
		
		I(i)=C*D;
		
		clf(i).w=w;
	end
	
	