%GLDBM_MULTI GLDB bottom-up, multi-class crit using Fisher separability
% 
%  W = GLDBM(A)
% 
% INPUT  
%  A    Training dataset
%
% OUTPUT
%  W    best bases mapping
%
% DESCRIPTION
% Generalised Best Bases mapping is a feature extraction technique for
% spectral datasets with two classes. Groups of adjacent features
% (wavelengths) are identified in the input dataset such that Fisher
% criterion is maximized in each group. Output dataset is created by
% projecting wavelengths in each group to the 1D Fisher subspace.
%
% REFERENCE 
% Kumar,Ghosh,Crawford: Classification of Hyperspectral Data using
% Best-bases Feature Extraction Algorithm
%
% SEE ALSO
% DATASETS, MAPPINGS

% Copyright: P. Paclik, pavel@ph.tn.tudelft.nl
% Faculty of Applied Physics, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

% $Id: gldbm_multi.m,v 1.4 2005/02/10 15:24:07 pavel Exp $

function w = gldbm_multi(a,arg2)

	% No arguments given: return untrained mapping.
		
	if (nargin < 1) | (isempty(a))
		w = mapping(mfilename);
		w = setname(w,'Best Bases mapping');
		return
	end

	if (~exist('arg2','var'))     % Second argument is not a mapping: training.

		isdataset(a);           % Assert that A is a dataset.

		[m,k,c] = getsize(a); M = classsizes(a);

		% counter with ten steps
		t=floor(m/10); 
		counts=t*ones(1,10);
		counts(1:mod(m,10))=counts(1:mod(m,10))+1;
		counts=cumsum(counts);
		
		m=0;              % level
		
		if mod(size(a,2),2)==0
			t=size(a,2);
		else
			t=size(a,2)-1;
		end
		l=1:2:t; % lower bounds of group-bands
		u=2:2:t; % upper bounds of group-bands
		u(end)=size(a,2); % asure that we use all bands
		Nm=length(l);

		% compute crit for each single band group:
		I=crit_eval(l,u,a);

		while 1
			% make possible merges:
			pl=l(1:Nm-1);
			pu=u(2:Nm);
			
			% eval criterium
			crit=crit_eval(pl,pu,a);
			[maxval,ind]=max(crit);
			while length(crit)>0
				% if the merge candidate with the
                                % highest crit.value yields smaller
                                % criterium then maximum of the original
                                % groups, choose 2nd best (3rd,... until
                                % the end
				if maxval<max(I([ind ind+1]))
					crit(ind)=[];
					[maxval,ind]=max(crit);
				else
					break
				end
			end
			if length(crit)==0
				% no merge candidate left, return what
                                % we have at the moment.
				[crit,clf]=crit_eval(l,u,a);

				% parameters
				pars.clf=clf; pars.l=l; pars.u=u; pars.info_iter=m;
				
				% get the output feature size from the clf
				featcount=0; for i=1:length(pars.clf), featcount=featcount + size(pars.clf(i).w,2); end
				w = mapping(mfilename,'trained',pars,getlablist(a),k,featcount);
				w = setname(w,'Best Bases mapping');
				
				return
			end
			
			% update crit vector
			I(ind)=maxval;
			I(ind+1)=[];

			u(ind)=u(ind+1);
			u(ind+1)=[];
			l(ind+1)=[];
			
			m=m+1;
			Nm=Nm-1;
			fprintf('.');
% $$$ 			if m==counts(1)
% $$$ 				counts(1)=[];
% $$$ 				fprintf('.');
% $$$ 			end

			if Nm==1
				% all the features were lumped together
				[crit,clf]=crit_eval(l,u,a);
				% parameters
				pars.clf=clf; pars.l=l; pars.u=u; pars.info_iter=m;
				% get the output feature size from the clf
				featcount=0; for i=1:length(pars.clf), featcount=featcount + size(pars.clf(i).w,2); end
				w = mapping(mfilename,'trained',pars,getlablist(a),k,featcount);
				w = setname(w,'Best Bases mapping');
				return; 
			end
		end

	else                      % Second argument is a a mapping: testing.

		w = arg2;
		pars = getdata(w);		 	% Unpack the mapping.
		[m,k] = getsize(a); 

		% create an output dataset
		out=[];
		for i=1:length(pars.clf)
			% get the subset of bands...
			t=+a(:,pars.l(i):pars.u(i));
			% ... and project it to 1D Fisher subspace
			out=[out t*pars.clf(i).w];
		end
	
		w = setdat(a,out);
	end

return


function [I,clf]=crit_eval(l,u,data)

	I=zeros(1,length(l));

	prior=getprior(data);
	clf=[];
	[m,k,c]=getsize(data);
	for k=1:length(l)

		Q=corrcoef(+data(:,l(k):u(k)));
		% correlation
		C=min(min( Q ));
		
		% turn within-class scatter into identity matrix
		w = klms(data(:,l(k):u(k)));
		b = data(:,l(k):u(k))*w;
		U = +meancov(b);
		d = ones(size(U,1));
		
		% Compute the weights E to be used in the modified between-scatter matrix G
		% E should diminish the influence of large distances D.
		e = ones(size(U,1));%0.5*erf(sqrt(d)/(2*sqrt(2))); 
		B = zeros(size(b,2));
		for j = 1:c
			for i=j+1:c
				% Marco-Loog Mapping
				B = B + prior(i)*prior(j)*e(i,j)*(U(j,:)-U(i,:))'*(U(j,:)-U(i,:))/d(i,j); 
				B = (B + B')/2;		% Avoid a numerical inaccuracy: cov. matrix should be symmetric!
			end
		end
		
		% criterion for the k-th feature group
		[F,V] = eig(B);
		[V,ind] = sort(-diag(V)); 
		fc=min(size(b,2),c)-1; % maximal number of features
		clf(k).w=F(:,ind(1:fc)); % projection matrix:
		I(k)=C*sum(-V(1:fc)); % criterion is a sum of eigenvalues
	end