% experimental: GLDB using top-down criterion (Log-odds)

%GLDBM_TD Generalised Best Bases (top-down) mapping for spectra
% 
%  W = GLDBM_TD(A)
% 
% INPUT  
%  A    Training dataset
%
% OUTPUT
%  W    best bases mapping
%
% DESCRIPTION 
% Generalised Best Bases mapping is a feature extraction technique for
% spectral datasets with two classes.  This function implements the
% top-down approach starting from a single group with all the wavelengths,
% recursively identifying the best possible split points.  Groups of
% adjacent features (wavelengths) are identified in the input dataset such
% that Fisher criterion is maximized in each group. Output dataset is
% created by projecting wavelengths in each group to the 1D Fisher
% subspace.  The procedure is defined for two class problems, if a
% multi-class dataset is supplied, a cell array of mappings will be
% computed.  Multi-class classifier, based on this routine, is implemented
% as PREXP algorithm ALG_GLDBM
%
% REFERENCE 
% Kumar,Ghosh,Crawford: Classification of Hyperspectral Data using
% Best-bases Feature Extraction Algorithm
%
% SEE ALSO
% DATASETS, MAPPINGS, GLDBM

% Copyright: P. Paclik, pavel@ph.tn.tudelft.nl
% Faculty of Applied Physics, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

% $Id: gldbm_td_prob.m,v 1.4 2005/02/10 15:24:07 pavel Exp $

function [w,crit,history] = gldbm_td_prob(a,arg2)

	% No arguments given: return untrained mapping.

	if (nargin < 1) | (isempty(a))
		w = mapping(mfilename);
		w = setname(w,'Best Bases mapping (top-down,log-odds)');
		return
	end

	if (~exist('arg2','var'))     % Second argument is not a mapping: training.

		isdataset(a);           % Assert that A is a dataset.

		[m,k,c] = getsize(a); M = classsizes(a);
		
		if c>2
			% for multi-class problem, create all pair-wise extractors
			w={};
			iter=[];
			for i=1:c-1
				for j=i+1:c
					fprintf(1,'(%d-%d',i,j);
					t=seldat(a,[i j]);
					eval(['w{i,j}=' mfilename '(t);']);
					fprintf(1,') ');
				end
			end
			return
		end

		% counter with ten steps
		t=floor(k/10); % usually, only 80% bands is processed
		counts=t*ones(1,10);
		counts(1:mod(m,10))=counts(1:mod(m,10))+1;
		counts=cumsum(counts);
		
		Q=corrcoef(+a);
		[U,G]=meancov(a);

		m=0;              % level
		Nm=size(a,2);
		
		% in top-down approach, each group of bands is defined by its limits (l,u),
		% flag saying if it should be processed further and a criterion value:

		bi=[];  % band info structure
		
		bi.l=1;  % lower bounds of group-bands
		bi.u=Nm; % upper bounds of group-bands
		bi.process=1; % should the group be processed?
		bi.crit=0;    % criterion values for each group

%		history={};
		
		% compute crit for each single band group:
%		bi.crit=crit_eval(l,u,a,bi);
		
		while 1
		    
		    % we'll go over all the groups and process the processable ones:
		    temp=find(bi.process==1);
		    gid=temp(1); % the 1st group in queue to be processed
%		    fprintf(1,' l=%d,u=%d ',l(gid),u(gid));
		    
		    % make possible merges:
		    % pl and pu refer to the possible left subgroup!
		    gc=bi.u(gid)-bi.l(gid)+1; % number of possible left subgroups
		    pl=repmat( bi.l(gid), 1, gc-1 ); % possible beginnings of the left new subgroup
		    pu=pl(1):bi.u(gid)-1;

		    % eval criterium for left subgroups:
		    crit1=crit_eval(pl,pu,a,bi);
		    
		    % eval criterion for right subgroups:
		    crit2=crit_eval(pu+1,repmat(bi.u(gid),1,gc-1),a,bi);
		    
		    maxval=max([crit1; crit2]);
		    [maxval,ind]=max(maxval);

		    newcrit=[crit1(ind) crit2(ind)];
		    
		    % pu(ind) is the best possible splitting index

		    % decide which of the two new subgroups should be processed further
		    newprocess=[0 0]; % by default: don't process
		    if crit1(ind)>bi.crit(gid) & ind>1
			% decompose the left subgroup
			newprocess(1)=1;
		    end
		    if crit2(ind)>bi.crit(gid) & gc-ind>1
			% decompose the left subgroup
			newprocess(2)=1;
		    end

		    % update high-level group definition:
		    if gid==1
			% we're splitting the firrst group
			bi.l=[ 1       pu(ind)+1 bi.l(2:end)];
			bi.u=[ pu(ind) bi.u(1)      bi.u(2:end)];
			bi.process=[newprocess bi.process(2:end)];
			bi.crit=[newcrit bi.crit(2:end)];
		    elseif gid==Nm
			% we're splitting the last group
			bi.l=[ bi.l(1:end-1) bi.l(gid) pu(ind)+1];
			bi.u=[ bi.u(1:end-1) pu(ind) Nm];
			bi.process=[bi.process(1:end-1) newprocess];
			bi.crit=[bi.crit(1:end-1) newcrit];
		    else
			% we're in the middle
			bi.l=[ bi.l(1:gid-1) bi.l(gid) pu(ind)+1 bi.l(gid+1:end)];
			bi.u=[ bi.u(1:gid-1) pu(ind) bi.u(gid)   bi.u(gid+1:end)];
			bi.process=[bi.process(1:gid-1) newprocess bi.process(gid+1:end)];
			bi.crit=[bi.crit(1:gid-1) newcrit bi.crit(gid+1:end)];			
		    end
		    fprintf(1,'(%d) ', sum(bi.process));
% $$$ 
% $$$ 		    % history:
% $$$ 			pars.l=bi.l; pars.u=bi.u; pars.info_iter=m;
% $$$ 			w = mapping(mfilename,'trained',pars,getlablist(a),k,length(pars.l));
% $$$ 			w = setname(w,'Best Bases mapping (top-down,log-odds)');
% $$$ 		    history{end+1}=w
		    
		    
		    if isempty(find(bi.process==1))
			% no candidates for progress left, return what
			% we have at the moment.
			%crit=crit_eval(bi.l,bi.u,a,bi);

			% parameters
			%pars.clf=clf; % (features == means are made on the spot)
			crit=bi.crit;
			pars.l=bi.l; pars.u=bi.u; pars.info_iter=m;
			w = mapping(mfilename,'trained',pars,getlablist(a),k,length(pars.l));
			w = setname(w,'Best Bases mapping (top-down,log-odds)');
			
			return
		    end
		end % while
	else % Second argument is a a mapping: testing.

		w = arg2;			
		pars = getdata(w); % Unpack the mapping.
		[m,k] = getsize(a); 

		% create an output dataset
		out=[];
		for i=1:length(pars.l)
			% get the mean of a subset of bands...
			t=mean(+a(:,pars.l(i):pars.u(i)),2);
			out=[out t];
		end
	
		w = setdat(a,out);
		w=setfeatlab(w,(1:size(w,2))');
	end

return

function crit=crit_eval(l,u,a,bi)
    
	crit=zeros(1,length(l));

	clf=[];
	for i=1:length(l)
	
	    b=a(:,l(i):u(i));
	    b=setdat(b,mean(+b,2)); % merge wavelengths somputing mean
	    
	    w=qdc(b); % estimate Gaussian parameters
	    
	    b=b*w; % get the probabilities
	    
	    b1=seldat(b,1); % class1
	    b2=seldat(b,2); % class2
	    
	    % log-odds criterion
	    crit(i)=sum(log((+b1(:,1))./((+b1(:,2))+eps)))/size(b1,1)+...
		    sum(log((+b2(:,2))./((+b2(:,1))+eps)))/size(b2,1);
	    
	end
    
    
    return

