% LIKNONC Liknon feature extractor and classifier
%
%     W=LIKNONC(A,C)
%
% INPUT
%   A   input dataset
%   C   optional C constant (tradeoff between misclassification and
%       sparseness), default C=1
%
% OUTPUT
%   W   Liknon classifier mapping
%
% DESCRIPTION
% Liknon is a feature extractor and classifier developed by Michael
% I.Jordan. By optimizing L1 norm, a sparse solution and linear
% classifier is obtained. For reference, look at:
%
% REFERENCE
%   Bhattacharyya, C., Grate, L.R., Rizki, A., Radisky, D.C., Molina, F.J.,
%   Jordan, M.I., Bissell, M.J. and Mian, I.S. "Simultaneous relevant
%   feature identification and classification in high-dimensional spaces:
%   application to molecular profiling data." Signal Processing (2002).
%

% $Id: liknonc.m,v 1.4 2005/02/10 15:24:07 pavel Exp $

function w=liknonc(a,m)

   if (nargin < 1) | (isempty(a)  & ~exist('m','var'))  
      % Definition: an untrained mapping.
      w = mapping(mfilename,{});
      w = setname(w,'Liknon classifier');

   elseif isempty(a) & isnumeric(m)
       
      w = mapping(mfilename,{m});
      w = setname(w,'Liknon classifier');
      
   elseif ~exist('m','var') | ~ismapping(m)
      
       if exist('m','var') & isnumeric(m) & length(m)==1
	   % we assume the parameter is a C constant
	   C=m;
       else
	   C=1; % default
       end
      % Training: W will be the trained mapping.
      [m,k,c] = getsize(a);

      [n,lablist]=classsizes(a);
      
      % prepare +1 / -1 labels
      y=[ones(n(1),1);-1*ones(n(2),1)];
      
      % run Michael I. Jordan's lpc
      [w,b]=lpc(+a,y,C);
      
      % select only non-zero dimensions
      ind=find(w~=0);
      w=w(ind);
      
      w=mapping(mfilename,'trained',{ind,w,b},a.lablist,k,c);
      
   else 

      % Execution of the mapping, W will be a dataset.
      v = +m;
      
      ind=v{1}; % indices of features to be used
      w=v{2}; % weight vector
      b=v{3}; % bias

      out=(+a(:,ind))*w+b;
      
      d=zeros(size(out,1),2);
      ind=find(out>0);
      d(ind,1)=out(ind);
      ind=setdiff(1:size(out,1),ind);
      d(ind,2)=-out(ind);
      
      % the best is minimum, we turn it by switching the sign:
      w=dataset(d,getlab(a),'featlab',getlab(m));
      
      w=setdata(a,d);
      w=setfeatlab(w,getlab(m));
      
   end
   
   return
   
% ----------------------------------------

function [a, bias] = lpc(X,Y,C)
%  Usage: [a, bias] = lpc(X,Y,C)
%
%  Parameters: X      - Training inputs
%              Y      - Training signs
%              C      - parameter
% this is the heart of the LIKNON system.  The matrix X and vector
% of labels Y are the input data.  This routine creates the linear
% programming matrix, calls the linear solver to solve it, and then
% maps the results back into the desired result vector a and scaler b.

% the problem being solved is find a hyperplane ax + b such that
% most of the vector a values are zero.  This is done by transforming
% it into an approximation that can be solved by linear programming.
% If we want the solution that has the fewest non-zero a components,
% that is a NP hard problem: min the L0 (L zero) norm of a, which is the
% count of non-zero components.  An approximation is to min the L1 norm,
% which is the sum of the absolute values: min (sum abs(a[i])).  This
% is what liknon solves.

% Min:  sum (abs(a[i])) + C * sum(slack[j]);
% subject to constraints:
%   y[j] * (a'x[j] + b) >= 1 - slack[j]
%   slack[j] >= 0.

% x[j] is the j'th row of the matrix X, y[j] is the j'th value of the
% experiment label (must be either +1 or -1).  There is one slack for
% each row.  We want the vector a, scalar b and vector slack as
% results.

% linear programming requires the variables to have limited range,
% such as the slack's all being not-negative.  This is a typical constraint.
% Since the a components can be anything, we need to transform the problem.
% The trick is splitting the a's and b into two parts, a "positive" and 
% "negative" part.  Only one of which will be non-zero at the end.

% so:
% let a[i] = u[i] - v[i], and b = b+ - b-.
% now the problem becomes:
% Min: sum (u[i] + v[i]) + C * sum(slack[j])
% subject to constraints:
%   y[j] * ((u[i] - v[i])'x[j] + b+ - b-) >= 1 - slack[j]
%   slack[j], u[i], v[i], b+, b- , >= 0.

% This is the problem being solved.

%This problem is published in a couple of papers:
% Bhattacharyya, C., Grate, L.R., Rizki, A., Radisky, D.C., Molina, F.J., 
% Jordan, M.I., Bissell, M.J. and Mian, I.S. "Simultaneous relevant feature
% identification and classification in high-dimensional spaces: application
% to molecular profiling data." Signal Processing (2002). 

% Grate, L.R., Bhattacharyya, C., Jordan, M.I. and Mian, I.S. "Integrated
% analysis of transcript profiling and protein sequence data." Mechanisms
% of Ageing and Development (2002).

% Grate, Bhattacharyya, Jordan, Mian. "simultaneous relevant feature
% identification and classification in high-dimensional spaces".  WABI2002

% 
  if (nargin <3) % check correct number of arguments
    help lpc 
  else

    [n, m]= size(X); % input data, n rows (examples), m cols (genes)

    epsilon = 1e-12; % values smaller will round to zero.
    I = eye(n); 
    % Construct the  matrix
%fprintf('Constructing %d %d\n',n,m);
    % Build the problem matrix. 
    H = zeros(n, 2*m +n+2);  
    for i = 1:n
        % build array for the constraints, see above equation.
        % fill array.  a is split into two parts, u,v.
        %               u       v        slacks     b+  b-
      H(i,:) = Y(i)*[ X(i,:), -X(i,:), Y(i)*I(i,:), 1, -1];  
    end
    rhs = ones(n,1);    %constraints right-hand-side, all 1's (since slacks moved over to left side.)
    f = ones(2*m,1);               % 1, multiply u, v
    f(2*m+1:2*m+n) = C*ones(n,1);  % C * slacks
    f(2*m+n+1) = 0;                % b+, b- are not part of the min equation.
    f(2*m+n+2) = 0; 

     vlb = zeros(2*m+n+2,1);      % lower bounds array: all >= 0

%fprintf('starting %s\n',datestr(now));
%st = cputime;

    % The linprog matlab call solves min f'x, subject to Hx <= rhs
    % since multiply by -1, subject to -Hx >= -rhs

    defaults = linprog('defaults');   
    defaults=optimset(defaults,'Display','none');
    [x,fval,exitflag,output,lambda] = linprog(f, -H, -rhs,[],[], vlb,[],[],defaults);
    
%fprintf('Execution time: %4.1f seconds\n',cputime - st);
    
   u = x(1:m);      % u are the "positive" halves of a
   v = x(m+1:2*m);  % v are the "negative" halves of a
   xi = x(2*m + n); % the "slacks" 
   bias = x(2*m + n + 1)-x(2*m+n+2);
   a = u - v;       % build a from the u, v vectors.
 %The problem is now solved.  But due to finite machine precision, there
 %exists noise in the solutions.  Smash all "too small to care" values
 %to zero.
   for i =1 :m
     if(abs(a(i)) < epsilon)
        a(i) = 0;
     end
   end
end    


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright (c) 1997-2002 The Regents of the University of California.
% All rights reserved.
% Permission is hereby granted, without written agreement and without
% license or royalty fees, to use, copy, modify, and distribute this
% software and its documentation for any purpose, provided that the above
% copyright notice and the following two paragraphs appear in all copies
% of this software.
%
% IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
% FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
% ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
% THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
% SUCH DAMAGE.
%
% THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
% MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
% PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
% CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
% ENHANCEMENTS, OR MODIFICATIONS.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
