%MAF Maximum autocorrelation mapping
% 
% 	[W,ALF] = MAF(A,N)
% 	[W,N] = MAF(A,ALF)
%
% INPUT
%    A    dataset
%    N    number of returned components
%    ALF  counterpart of fraction of preserved variance in PCA
%
% OUTPUT
%    W    traned MAF mapping
%
% DESCRIPTION
% This is a special purpose version of PCA for images stored as features
% in the dataset. The covariance matrix is slightly modified such that
% the covariances are computed for the one-pixel shifted images.
% Futher, transormed data are constraind to be univariated.
% As a result the transormation maximizes image autocorrelation.

% This code comes from prtools 3.1.6:
% Copyright: R.P.W. Duin, duin@ph.tn.tudelft.nl
% Faculty of Applied Physics, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

% $Id: maf.m,v 1.7 2007/01/26 10:16:46 serguei Exp $

function [w,q,acov] = maf(a,alf)

	if nargin < 2 | isempty(alf), alf = inf; end
	if nargin < 1 | isempty(a)
		w = mapping('maf',alf); return
	end

	isdataset(a);
	isfeatim(a);
	
	[m,k,c]=getsize(a);
	
	u = [];
  % The reduction of dimensionality
  % is not really neccessary: it makes data smaller but for the cost 
  % of extra svd on all data, cov matrix computation and eig on it. 
  % If we have enough memory, it is faster to compute acov on all the data and then do eig.
  %if m <= k
	%	u = reducm(a);
	%	a = a*u;
	%	korg = k;
	%	[m,k] = size(a);
  % end

  G = imcov(data2im(a));

	[F V] = eig(G);
	[v,I] = sort(-diag(V));
	if alf == inf
		n = ni;
		q = ni;
	elseif alf >= 1
		n = alf;
		if n > k
			error('Illegal dimensionality requested');
		end
		q = sum(v(1:n))/sum(v);
		I = I(1:n);
	elseif alf > 0
		vv = v'*triu(ones(k,k)) / sum(v) - alf;
		J = find(vv > 0);
		n = J(1); q = n;
		I = I(1:n);
	elseif alf == 0
		w = ones(1,k);
		ww = v'*triu(ones(k,k)) / sum(v);
		w(1:length(ww)) = ww;
		return
	elseif alf > -1
		alf = abs(alf);
		v = flipud(v); I = flipud(I);
		vv = v'*triu(ones(k,k)) / sum(v) - alf;
		J = find(vv > 0);
		n = J(1)-1; q = n;
		I = I(1:n);
	else
		n = abs(alf);
		v = flipud(v); I = flipud(I);
		if n > k
			error('Illegal dimensionality requested');
		end
		q = sum(v(1:n));
		sv = sum(v);
		if sv ~= 0, q = q/sv; end
		I = I(1:n);
	end
	
  acov = abs(v);
  
  F = F(:,I);
  
  if ~isempty(u)
	  offset = u.data.offset*F;  
    F = u.data.rot*F; 
  else
    offset = -mean(a*F);        
  end
  
  w = affine(F,offset);

  return

%IMCOV Image covariance
%
%	c = imcov(a)

function c = imcov(a)
	[m,n,k] = size(a);
	g = mean(reshape(a,m*n,k));
	J = bord(reshape(1:m*n,m,n),NaN);
  
  %c = zeros(k,k);
  %for i=1:k
	%  a1 = a(:,:,i) - g(i);
	%	 for j=i:k
	%	   a2 = a(:,:,j) - g(j);
	%		 cc = mean(mean(a1.*a2(J(1:m,2:n+1)) + a1.*a2(J(3:m+2,2:n+1))));
	%		 cc = cc + mean(mean(a1.*a2(J(2:m+1,1:n)) + a1.*a2(J(2:m+1,3:n+2))));
	%		 cc = cc/4;
	%		 c(i,j) = cc; c(j,i) = cc;
  %  end
  %end

  % if we have enough memory we can do vectorization
  a = permute(a,[3, 1, 2])-repmat(g',[1 m n]);
  c = a(:,:)*(a(:,J(1:m,2:n+1))' + a(:,J(3:m+2,2:n+1))' + a(:,J(2:m+1,1:n))' + a(:,J(2:m+1,3:n+2))')/4;
  c = (c+c')/2;
  return

% C = bord(A,n,m)
% Puts a border of width m (default m=1) around image A
% and gives it value n. If n = NaN: mirror image values.
function C = bord(A,n,m);
	if nargin == 2; m=1; end
	[x,y] = size(A);
	if m > min(x,y)
		mm = min(x,y);
		C = bord(A,n,mm);
		C = bord(C,n,m-mm);
		return
	end
	if isnan(n)
		C = [A(:,m:-1:1),A,A(:,y:-1:y-m+1)];
		C = [C(m:-1:1,:);C;C(x:-1:x-m+1,:)];
	else
		bx = ones(x,m)*n;
		by = ones(m,y+2*m)*n;
		C = [by;[bx,A,bx];by];
	end
	return
	
