%MINDISTC Minimum distance classifier
% 
%  W=MINDISTC(A)
% 
% INPUT  
%  A    Training dataset with dissimilarities to training prototypes
%
% OUTPUT
%  W    Minimum distance classifier mapping
%
% DESCRIPTION
% Minimum distance classifier measures minimum distance to a prototype from
% each class. During training, only information about the classes is
% stored, but no data or parameters. During execution, MINDISTC expects
% that dataset with dissimilarities to training prototypes is supplied.
% For each class, minimum dissimilarity is returned for each class. 
% Classifier outrput is a negative dissimilarity i.e. similarity
% apropritate for prtools routines such as TESTC or CLASSD, may be used
% directly.
%
% SEE ALSO
% DATASETS, MAPPINGS

% Copyright: P. Paclik, pavel@@ph.tn.tudelft.nl
% Faculty of Applied Physics, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

% $Id: mindistc.m,v 1.12 2005/04/08 12:57:21 pavel Exp $

function w=mindistc(a,arg2)
	
	% No arguments given: return untrained mapping.

	if (nargin < 1) | (isempty(a))
		w = mapping(mfilename,{});
		w = setname(w,'Minimum distance classifier');
		return
	end

	if (~exist('arg2','var'))     % Second argument is not a mapping: training.
		
		% in training, we do nothing
		[m,k,c] = getsize(a);     % size of the training set		
		data=[];
		w = mapping(mfilename,'trained',data,getlablist(a),k,c);
		w = setname(w,'Minimum distance classifier');
		
	else
		
		% execution of minimum distance classifier
		
		% input dataset is supposed to be a set of
                % dissimilarities to training prototypes

		ll=getlab(arg2); % lablist from the trained classifier
		flab=getfeatlab(a); % feature labels

		if ~strcmp(class(ll),class(flab))
		    error(['lablist of the classifier and feature labels' ...
			   ' of the mapping are of different format']);
		end
		
		% nlab indices of a colums
		featind=matchlablist(flab,ll);
		featindll=unique(featind); % unique class ids
		
		out=zeros(size(a,1),size(ll,1));

		for i=1:length(featindll)
			
			ind=find(featind==featindll(i));
			
			% we find minimum distance for each test object
			m=min(+a(:,ind),[],2);
			
			out(:,i)=m;
			
		end

		out=-out;
		
		% first, we create a new dataset with original labels
		% (we cannot setdat because the output has different
                % feature labels)
                w=dataset(out,getlab(a));
		
		% previous operation sorted the lablist into alphabetic
                % order!
		
		% now, we set the correct feature labels for the output
                % columns: sorted lablist is already in w, the feature
                % labels are unique class names from the original
                % training set lablist ll
                w=dataset(out,getlab(w),'lablist',w.lablist,'featlab', ...
			  ll(featindll,:),'prior',a.prior,'user',a.user);
		w.ident=a.ident;
	end
	
	return
