%OPA orthogonal projection approach to the selection of "pure" objects (key set of objects)
%
%  [Y,SEL_OBJ_IND,SEL_OBJ_DIS,DIS,ERR] = OPS(X,OPT)
%
% INPUT 
%   X             [M -by- K] double matrix  
%
%   OPT.   
%    MAX_NUM      the max number of objects to select, INF means selecting as many objects as possible,
%                 until the rank of dataset is exausted; at least one variable will be always selected; by delault = INF
%    EPS          stopping criteion; algorithm stops ERR < EPS, such objects are not included into the set; by default = 1e-2
%    VERBOSE      force/disable messages; if PRPROGRESS is available: PRPROGRESS(VERBOSE,['%s\n'],MSG) will be run, 
%                 if PRPROGRESS is not available: VERBOSE == [] or 0 -- no messages, VERBOSE ~= 0 runs DISP(MSG); 
%                 by default = []
%
%
% OUTPUT 
%   Y              [M_SEL -by- K] dataset containing selected object
%   SEL_OBJ_IND    [M_SEL -by- 1] vector of selected objects indices 
%   SEL_OBJ_DIS    [M_SEL -by- 2] two vectors of selected objects dissimilarities; 
%                  in the first column are absolute dissims, and in the second are relative dissim
%   DIS            [M -by- M_SEL -by- 2] two matrices which columns are dissimilarities profiles; 
%                  each column contains dissimilarities of all objects at particular selection step;
%                  the first matrix contains abs dissim, the second one containes relative dissim
%   ERR            [M_SEL -by- 1] relative squared error of the data projection to the subspace spanned by the select objects
%
% DESRIPTION
% Low level routine.
% Implementation of the OPA algorithm as described in
% "Orthogonal Projection Approach Applied to Peak Purity Asessment"  
%  by F. C. Snachez, J. Toft, B. van den Bogaert, D. L. Massart, Anla. Chem. 68, 1, 79-85, 1996 
%
% Absolute dissim measure  abs_diss_(n+1) = ||dx||^2*det(Y_n Y_n^T)
% Relative dissim measure  rel_diss_(n+1) = ||dx||^2
%
% Here, Y_n is the set of n selected objects (rows), which are normalized to have a unit length, dx is the part of 
% an object x which is (part) orthogonal to span(Y). If n=0 (diss(1)) then the dissimilarity to the mean object is measured.
%
% SEE ALSO
% OPAM, OPAPS, SIMPLISMA, VARIMAX, ALS

% Copyright: S.Verzakov, s.verzakov@ewi.tudelft.nl 
% Faculty EWI, Delft University of Technology
% P.O. Box 5031, 2600 GA Delft, The Netherlands


% $Id: opa.m,v 1.9 2006/03/15 12:17:15 serguei Exp $

function [Y, sel_obj_ind, sel_obj_dis, dis, err] = opa(X,Options)

if nargin < 2
  Options = [];
end

DefOptions.max_num    = inf;
DefOptions.eps        = 1e-2;
DefOptions.verbose    = [];
Options = updstruct(DefOptions, Options);

%----------------
prprog = exist('prprogress');
if ~prprog & isempty(Options.verbose)
  Options.verbose = 0;
end
%----------------


[N, d] = size(X);
Options.max_num = min([Options.max_num, N, d]);

Y_m = mean(X,1);
O = Y_m/sqrt(Y_m*Y_m');

X_ss = sum(X.*X,2); 
SS = sum(X_ss);

dis = X_ss - (X*O').^2;
%err0 = sum(dis)/SS;
dis = cat(3,dis,dis);
[sel_obj_dis, ind] = max(dis(:,1,2));
sel_obj_dis = [sel_obj_dis, sel_obj_dis];

O = X(ind,:)/sqrt(X_ss(ind));

Y = X(ind,:);
X(ind,:)    = [];
X_ss(ind,:) = [];
dX_ss = X_ss;

sel_obj_ind = ind;
obj_ind = [1:N]';
obj_ind(ind) = [];

Det_factor = 1;
Det = Det_factor;

dX_ss = dX_ss - (X*O(end,:)').^2;
err = sum(dX_ss)/SS;

Msg = ['step 1; max dissim to the mean sample is dis(' num2str(ind) ')=' num2str(sel_obj_dis(1)) ... 
       '; err=' num2str(err)];
if prprog
  prprogress(Options.verbose,['%s\n'],Msg);
elseif Options.verbose
  disp(['opa: ' Msg]);
end  

for i=2:Options.max_num
  if err(i-1) < Options.eps
    break;
  end

  dis(obj_ind,i,2) = dX_ss;  
  [sel_obj_dis(i,2), ind] = max(dis(obj_ind,i,2));
  Det_factor = sel_obj_dis(i,2)/X_ss(ind);

  dis(obj_ind,i,1) = dis(obj_ind,i,2)*Det;
  sel_obj_dis(i,1) = sel_obj_dis(i,2)*Det;
  Det = Det_factor*Det;
  
  O = [O; (X(ind,:) - (X(ind,:)*O')*O)/sqrt(dX_ss(ind))];
  Y = [Y; X(ind,:)];
  X(ind,:)     = [];
  X_ss(ind,:)  = [];
  dX_ss(ind)   = [];
  
  sel_obj_ind  = [sel_obj_ind; obj_ind(ind)];
  obj_ind(ind) = [];
  
  dX_ss = dX_ss - (X*O(end,:)').^2;
  err(i,1) = sum(dX_ss)/SS;
  
  Msg = ['step ' num2str(i) ...
         '; max dissim to the current set of sample(s) are' ...
         ' abs diss(' num2str(sel_obj_ind(i)) ')=' num2str(sel_obj_dis(i,1)) ...
         ' rel diss(' num2str(sel_obj_ind(i)) ')=' num2str(sel_obj_dis(i,2)) ...
         '; err=' num2str(err(i))];
 
  if prprog
    prprogress(Options.verbose,['%s\n'],Msg);
  elseif Options.verbose
    disp(['opa: ' Msg]);
  end  
end

return





















