%pca_crit Calculation of critical levels for PCA
%
%  CritLevel = pca_crit(conf_level,lambda,nPC_model,Options)
%  CritLevel = pca_crit(conf_level,lambda,nPC_model,Options,Stat,ecv_only)
%
% INPUT 
%   conf_level  confidence level(s) (1-alpha error) (conf_level=0.95 by default);
%               conf_level van be row, vector or matrix
%   lambda      [1 -by- nPC]  first nPC largest eigenvalues 
%   nPC_model   the number of PC's in the model for which crit levels will be calculated,
%               nPC_model <= length(lambda)
%   Options     structure returned by pca_train
%   Stat        structure returned by pca_stat; it is used to calculate empirical critical levels
%   ecv_only    calculate only empirical critical values; by default = 0
%
%
%   mean_centering was (mean_centering=1) or was not (mean_centering=0) data mean centered
%                    in the feature space (mean_centering=1 by default)
% OUTPUT
%   CritLevel.            structure containig critical levels calculated
%    conf_level            confidence level(s) used for calculations
%    nPC_model             number of the scores retained in the model 
%    nPC                   total number of the availabel PCs
%                          PC(nPC_model+1) ... PC(nPC) were used to calculate  T^2_rest
%    Q, Q_alt, Q_emp       critical levels for Euclidian distance to the model plane
%    T2, T2_emp            critical levels for Mahalanobis distance in the model plane
%    T2_rest, T2_rest_emp  critical levels for (partial) Mahalanobis distance to the model plane
%   
%
% DESCRIPTION
%   
% 
% SEE ALSO
% pca_apply

% Copyright: S. Verzakov, serguei@ph.tn.tudelft.nl
% Faculty of Applied Sciences, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

% $Id: pca_crit.m,v 1.7 2005/05/05 14:35:55 serguei Exp $



function CritLevel = pca_crit(conf_level,lambda,nPC_model,Options,Stat,ecv_only)

if nargin < 5
  Stat = [];
end

if nargin < 6
  ecv_only = 0;
end

N   = Options.N;
M   = Options.M;
nPC = Options.nPC;
a   = Options.rank;

if nPC_model > nPC
  error('The number of availabel PC''s is smaller than the number of the PS''s retaind in the model');
end

%--- Q-statistics critical level
if nPC_model == a
  if ~ecv_only
    CritLevel.Q=zeros(size(conf_level));
    CritLevel.Q_alt = zeros(size(conf_level));
  end
  
  if ~isempty(Stat) & isfield(Stat,'Q')
    CritLevel.Q_emp = zeros(size(conf_level));  
  end  
  
else
  if ~ecv_only
    lambda_model = lambda(1:nPC_model);
    Th1 = Options.Tr1 - sum(lambda_model);
    Th2 = Options.Tr2 - sum(lambda_model.^2);
    Th3 = Options.Tr3 - sum(lambda_model.^3);
    
    c  = sqrt(2)*erfinv(2*conf_level - 1); % == norminv(conf_level);
    h0 = 1 - 2*Th1*Th3/(3*Th2*Th2);
    CritLevel.Q = Th1*( c*(h0*sqrt(2*Th2)/Th1) + (1 + Th2*h0*(h0-1)/(Th1*Th1)) ).^(1/h0);
    
    try
      c = finv(conf_level,a-nPC_model,(a-nPC_model)*(N-nPC_model-1));
      %% PP!!
      %  if (N-nPC_model-1)==0, keyboard; end
      CritLevel.Q_alt = (M*Th1/(N-nPC_model-1))*c;
    catch
      WarnMsg = sprintf(['CritLevel.Q_alt cannot be calculated, set to nan:\n' lasterr]);
      if exist('prwarning')
        prwarning(2, WarnMsg);
      else
        warning(WarnMsg);
      end
      CritLevel.Q_alt = repmat(nan,size(conf_level));  
    end
  end
  if ~isempty(Stat) & isfield(Stat,'Q')
    %CritLevel.Q_emp = prctile(Stat.Q,100*conf_level(:));
    CritLevel.Q_emp = get_percentile(Stat.Q,conf_level(:));
    CritLevel.Q_emp = reshape(CritLevel.Q_emp,size(conf_level));  
  end  
end

%--- T2-statistics critical level
if ~ecv_only
  try
    c = finv(conf_level,nPC_model,N-nPC_model);
    CritLevel.T2 = (M*nPC_model/(N-nPC_model))*c;
    
  catch
    WarnMsg = sprintf(['CritLevel.T2 cannot be calculated, set to nan:\n' lasterr]);
    if exist('prwarning')
      prwarning(2, WarnMsg);
    else
      warning(WarnMsg);
    end
    CritLevel.T2 = repmat(nan,size(conf_level));  
  end
end

if ~isempty(Stat) & isfield(Stat,'T2')
  %CritLevel.T2_emp = prctile(Stat.T2,100*conf_level(:));
  CritLevel.T2_emp = get_percentile(Stat.T2,conf_level(:));
  CritLevel.T2_emp = reshape(CritLevel.T2_emp,size(conf_level));  
end        

%------ T2_rest-statistics critical level
if nPC_model < nPC
  if ~ecv_only
    try
      c = finv(alpha_level,nPC-nPC_model,N-nPC+nPC_model);
      CritLevel.T2_rest = (M*(nPC-nPC_model)/(N-nPC+nPC_model))*c;
      
    catch
      WarnMsg = sprintf(['CritLevel.T2_rest cannot be calculated, set to nan:\n' lasterr]);
      if exist('prwarning')
        prwarning(2, WarnMsg);
      else
        warning(WarnMsg);
      end
      CritLevel.T2_rest = repmat(nan,size(conf_level));  
    end
  end  
  
  if ~isempty(Stat)  & isfield(Stat,'T2_rest')
    %CritLevel.T2_rest_emp = prctile(Stat.T2_rest,100*conf_level(:));
    CritLevel.T2_rest_emp = get_percentile(Stat.T2_rest,conf_level(:));
    CritLevel.T2_rest_emp = reshape(CritLevel.T2_rest_emp,size(conf_level));  
  end
else
  if ~ecv_only
    WarnMsg = sprintf(['The number of PC''s availabel is equal to the nubmer of the PC''s in model.\n'...
        'CritLevel.T2_rest cannot be calculated, set to nan\n']);
    if exist('prwarning')
      prwarning(2, WarnMsg);
    else
      warning(WarnMsg);
    end
    CritLevel.T2_rest = repmat(nan,size(conf_level));  
  end
  
  if ~isempty(Stat) & isfield(Stat,'T2_rest')  
    WarnMsg = sprintf(['The number of PC''s availabel is equal to the nubmer of the PC''s in model.\n'...
        'CritLevel.T2_rest_emp cannot be calculated, set to nan\n']);
    if exist('prwarning')
      prwarning(2, WarnMsg);
    else
      warning(WarnMsg);
    end
    CritLevel.T2_rest_emp = repmat(nan,size(conf_level));  
  end  
end

return;

