%pca_npc Estimation of the significant number of PCs
%
%  [Out,T,P,lambda,PCAOptions] = pca_npc(X,Options)
%
% INPUT 
%  X             [N -by- d] training input data matrix, N samples, d variables
%
%  Options.
%   M            scatter matrix normalization: C = X'*X/M
%                by default M = N-1 (so if columns of X were mean-centered, C is the unbiased estimation
%                of the covariance matrix)
%   X_centering  do nothing (==[] or 0), do mean centering (==nan), center around some vaue v (==v); by default  ==[]
%   X_scaling    do nothing (==[] or 1), divide each col by std (=nan), divide by some v (=v); by default =[]
%
%   nPC_em       nPC esimation method: 'PEV', 'MI', 'F', 'KG' , 'PRESS' or
%                cell array containing any combination  of this strings; 
%                'ALL' is the shortcut for {'PEV', 'MI', 'F', 'KG', 'PRESS'}; 
%                'NOPRESS' is the shortcut for the {'PEV', 'MI', 'F', 'KG'};
%                by default 'NOPRESS'
%   PEV          part of explained varince; according to this criteria nPC
%                is the minimum number of PC such that the portion of explained variance is >= pev; 
%                by default 0.95; if mean centering was not performed then pev defines the level 
%                not for the explained variance but for the explained sum of squares; 
%   F_cl         the confidence level for Malinowski F-test
%
%
% OUTPUT
%  Out. structure containing the results
%    max_nPC    maximum possible number of PCs
%    PEV        [1 -by- max_nPC] PEV(i) is the part of variance explained by i PCs
%    nPC_PEV    number of PC according to PEV citerion
%    MI         [1 -by- max_nPC] MI(i) is the Malinowski indicator for the model with i PCs
%    nPC_MI     number of PC according to MI 
%    F          [1 -by- max_nPC] F(i) is the Malinowski F-ratio for the model with i PCs
%    nPC_F      number of PC according to F
%    KG         Kaiser-Guttman criterion (lambda/sum(lambda))
%    nPC_KG     number of PC according to KG : largets i such KG(i) > 1
%    PRESS      predicted error sum of squares (leave-one-out)
%    nPC_PRESS  ...
%
% DESCRIPTION
%
% SEE ALSO
% pca_apply

% Copyright: S. Verzakov, serguei@ph.tn.tudelft.nl
% Faculty of Applied Sciences, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

% $Id: pca_npc.m,v 1.5 2005/05/05 14:35:55 serguei Exp $


function [Out,T,P,lambda,PCAOptions] = pca_npc(X,Options)

if nargin < 2
  Options  = [];
end


DefaultOptions.nPC_em = 'NOPRESS';
DefaultOptions.PEV = 0.95;
DefaultOptions.F_cl = 0.95;

Options = updstruct(DefaultOptions, Options);


if ischar(Options.nPC_em)
  switch upper(Options.nPC_em)
  case 'ALL'
    Options.nPC_em = {'PEV', 'MI', 'F', 'KG', 'PRESS'};
  case 'NOPRESS'
    Options.nPC_em = {'PEV', 'MI', 'F', 'KG'};
  otherwise

  end
end

DoPEV   = any(strcmpi('PEV',Options.nPC_em));
DoMI    = any(strcmpi('MI',Options.nPC_em));
DoF     = any(strcmpi('F',Options.nPC_em));
DoKG    = any(strcmpi('KG',Options.nPC_em));
DoPRESS = any(strcmpi('PRESS',Options.nPC_em));

[N,d] = size(X);

Options.nPC = inf;

% Out = [];
%if DoPEV | DoMI | DoF | DoKG
[T,P,lambda,PCAOptions] = pca_train(X,Options);
Out.nPC_max = PCAOptions.nPC;
%end  


% PEV 
if DoPEV
  Out.PEV = PCAOptions.PEV;
  Out.nPC_PEV = min(find(PCAOptions.PEV >= Options.PEV));
end

% MI
if DoMI
  d1 = max(N,d);
  d2 = min(N,d);
  %d1 = min(N,d);
  %d2 = max(N,d);
  
  if Out.nPC_max == d2
    max_ind = Out.nPC_max-1;
    RE(Out.nPC_max) = 0;
    Out.MI(Out.nPC_max) = 0;
  else
    max_ind = Out.nPC_max;
  end
  RE(1:max_ind) = sqrt((PCAOptions.Tr1 - cumsum(lambda(1:max_ind)))./((d1-1).*(d2-(1:max_ind))));
  Out.MI(1:max_ind) = RE(1:max_ind)./((d2-(1:max_ind)).^2);
  if Out.nPC_max == PCAOptions.rank
    max_ind = Out.nPC_max-1;
  else
    max_ind = Out.nPC_max;
  end
  [MI_val, Out.nPC_MI] = min(Out.MI(1:max_ind));
end  

% F
if DoF
  REV = lambda./((d-(1:Out.nPC_max)+1) .* (N-(1:Out.nPC_max)+1));
  %avr_REV = fliplr(cumsum(fliplr(REV(2:end))));
  avr_REV = fliplr(cumsum(fliplr(REV(2:end))))./(Out.nPC_max-(1:(Out.nPC_max-1)));
  F_crit = finv(Options.F_cl,1,[(Out.nPC_max-(1:(Out.nPC_max-1)))]);
  Out.F = (REV(1:(end-1)) ./ avr_REV) ./ F_crit;
  Out.nPC_F = min(find(Out.F <= 1))-1;
end

%KG
if DoKG
  Out.KG = lambda(1:Out.nPC_max)/(PCAOptions.Tr1/d);
  Out.nPC_KG = min(find(Out.KG <= 1)) - 1;
end  

if DoPRESS
  if ~isempty(Out) & isfield(Out,'nPC_max')  
    nPC = Out.nPC_max; 
  else  
    nPC = min(N,d); 
  end

  Out.PRESS = zeros(N,nPC);  
  ind_test = 1:N;
  for i=1:N
    X_train = X;
    X_train(ind_test(i),:) = [];
    X_test = X(ind_test(i),:);
 
    [T_train,P_PRESS,lambda_PRESS,PCAOptions_PRESS] = pca_train(X_train,Options);
    T_test = pca_apply(X_test,P_PRESS,PCAOptions_PRESS);  
    X_test = prepro(X_test,PCAOptions_PRESS.X_centering,PCAOptions_PRESS.X_scaling);

    for j=1:PCAOptions_PRESS.nPC
      X_test = X_test - T_test(:,j)*P_PRESS(:,j)';
      Out.PRESS(ind_test(i),j) = X_test*X_test';
    end
    %Out.PRESS(ind_test(i),:) = cumsum((T_test(:,1:PCAOptions_PRESS.nPC).^2)./lambda_PRESS(1:PCAOptions_PRESS.nPC));

    nPC = min(PCAOptions_PRESS.nPC,nPC);
  end

  if ~isfield(Out,'nPC_max')  
    Out.nPC_max = nPC; 
  end

  Out.PRESS = sum(Out.PRESS,1);
  Out.PRESS = Out.PRESS(1,1:nPC);
  [PRESS_val, Out.nPC_PRESS] = min(Out.PRESS);
  %keyboard

%else
%  Out.PRESS = repmat(nan,[1, Out.nPC_max]);
%  Out.nPC_PRESS = nan;
end



return;

