%pca_train Principal Component Analysis (training)
%  
%  [T, P, lambda, Options] = pca_train(X)
%  [T, P, lambda, Options] = pca_train(X,Options)
%
% INPUT 
%  X             [N -by- d] training input data matrix, N samples, d variables
%
%  Options.
%   M            scatter matrix normalization: C = X'*X/M
%                by default M = N-1 (so if columns of X were mean-centered, C is the unbiased estimation
%                of the covariance matrix)
%   X_centering  do nothing (==[] or 0), do mean centering (==nan), center around some vaue v (==v); by default  ==[]
%   X_scaling    do nothing (==[] or 1), divide each col by std (=nan), divide by some v (=v); by default =[]
%   nPC          the number of principal components to return; by default inf;
%                nPC==inf means nPC==rank(preprocessed X);
%                nPC==nan means autodetection according to nPC_em
%   nPC_em       nPC esimation method: 'PEV', 'MI', 'F', 'KG', 'PRESS'; by default 'PEV'     
%   PEV          part of explained varince; if nPC==nan  and nPC_em is 'PEV' then the number of 
%                principal components to return is the minimum number of PC such that the portion
%                of explained variance is >= PEV; by default 0.95
%                if mean centering was not performed then PEV defines the level 
%                not for the explained variance but for the explained sum of squares; 
%   F_cl         confidence level  for Malinowski F-test, if nPC==nan  and nPC_em is 'F'  
%                then the F-test with such a conf. level will be used to estimate the number of PCs 
%  
% OUTPUT
%  T       [N -by- nPC]  scores   (transformed (preprocessed) X)
%  P       [X -by- n]    loadings (new axis in old coordinats)
%  lambda  [1 -by- nPC]  first nPC largest eigenvalues of X'*X/M (covariance matrix) of (preprocessed) X
%
% Options. contains
%   1) the same fields as in the input, but the values can be changed 
%      (e.g. after mean centering X_centering = mean(X,1); nPC==the number of retained components;
%      PEV==the portion of explained variance)
%   2) N, d are the sizes of the training set X
%   3) rank is the rank of the training set X
%   4) Tr1 = sum(lambda), Tr2 = sum(lambda.^2), Tr3 = sum(lambda.^3), 
%      where sum goes over all nonzero lambdas (not only over the first nPC ones) 
%
%
% DESCRIPTION
% Performs PCA. 
%
% Relations between matrices (X is assumed to be preprocessed):
%
% [X'*X/M]*P = P*diag(lambda)
% [X*X'/M]*T = T*diag(lambda)
% P'*P = eye(nPC)
% T'*T/M = diag(lambda)
% X = T*P' + Err
% T = X*P
% P*diag(lambda)*M = X'*T;
% explvar = sum(lambda)/trace(X'*X/M)
%  
%
% SEE ALSO
% pca_apply

% Copyright: S. Verzakov, serguei@ph.tn.tudelft.nl
% Faculty of Applied Sciences, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

% $Id: pca_train.m,v 1.9 2005/05/05 14:35:55 serguei Exp $

function [T, P, lambda, Options] = pca_train(X,Options)

if nargin < 2
  Options  = [];
end

[N, d] = size(X);

Options.N = N;
Options.d = d;

DefaultOptions.M = N-1;
DefaultOptions.X_centering = [];
DefaultOptions.X_scaling = [];
DefaultOptions.nPC = inf;
DefaultOptions.nPC_em = 'PEV';
DefaultOptions.PEV = 0.95;
DefaultOptions.F_cl = 0.95;

Options = updstruct(DefaultOptions, Options);
max_nPC = min(N,d);

if max_nPC == 0
  error('The maximum possible PC number is 0.');  
end

if ~isnan(Options.nPC)
  [X, Options.X_centering, Options.X_scaling] = prepro(X, Options.X_centering, Options.X_scaling);

  Options.nPC_em = '';
  %if isinf(Options.nPC)
  %  Options.nPC = max_nPC; 
  %end

  %if max_nPC < Options.nPC
  %  WarnMsg = sprintf(['The ordered number of PC''s (%d) is greater than\n' ... 
  %                   'the maximum possible number of PC''s corresponding\n' ...
  %                  ' to the non-zero eigenvalues(%d).\n' ... 
  %                   'Options.nPC set to %d'],Options.nPC,max_nPC,max_nPC);
  %  if exist('prwarning')
  %    prwarning(1, WarnMsg);
  %  else
  %    warning(WarnMsg);
  %  end
  %  Options.nPC = max_nPC;
  %end  


  if N >= d
    [P,lambda,P] = svd(X'*X);
    lambda = diag(lambda)';    
    %Options.rank = nnz(lambda);
    Options.rank = nnz(lambda > N*lambda(1)*1e-16);
    lambda(Options.rank+1:end) = 0;
    PEV = cumsum(lambda);
    PEV = PEV/PEV(end);    
    nPC = min(Options.rank, Options.nPC);
    P = P(:,1:nPC);
    T = X*P;

  else
    [T,lambda,T] = svd(X*X');
    lambda = diag(lambda)';  
    %Options.rank = nnz(lambda);
    Options.rank = nnz(lambda > d*lambda(1)*1e-16);
    lambda(Options.rank+1:end) = 0;
    PEV = cumsum(lambda);
    PEV = PEV/PEV(end);    
    nPC = min(Options.rank, Options.nPC);
    T = T(:,1:nPC).*repmat(sqrt(lambda(1:nPC)),[N, 1]);
    P = (X'*T)./repmat(lambda(1:nPC),[d, 1]);
  end

  if nPC < Options.nPC
    if ~isinf(Options.nPC)
      WarnMsg = sprintf(['The ordered number of PC(%d) is greater than the number of PC\n' ... 
                         'corresponding to the non-zero eigenvalues(%d).\n' ... 
                         'Options.nPC set to %d'],Options.nPC,nPC,nPC);
      if exist('prwarning')
        prwarning(1, WarnMsg);
      else
        warning(WarnMsg);
      end
    end  
    Options.nPC = nPC;
  end

  Options.PEV = PEV(1:Options.nPC);    
  lambda      = lambda/Options.M;    
  Options.Tr1 = sum(lambda);
  Options.Tr2 = sum(lambda.^2);
  Options.Tr3 = sum(lambda.^3);
  lambda      = lambda(1:Options.nPC);    

else  
  nPC_em = Options.nPC_em;
  [Out,T,P,lambda,Options] = pca_npc(X,Options);
  Options.nPC = eval(['Out.nPC_' nPC_em]);
  T = T(:,1:Options.nPC);
  P = P(:,1:Options.nPC);
  Options.PEV = Options.PEV(1:Options.nPC);    
  lambda      = lambda(1:Options.nPC);    
  
%  if N >= d
%    [P,lambda] = svd(X'*X);
%    lambda = diag(lambda)';    
%    PEV = cumsum(lambda);
%    PEV = PEV/PEV(end);    
%    Options.rank = nnz(lambda);
%    Options.nPC = min(find(PEV >= Options.PEV));
%    P = P(:,1:Options.nPC);
%    T = X*P;
%
%  else
%    [T,lambda] = svd(X*X');
%    lambda = diag(lambda)';  
%    PEV = cumsum(lambda);
%    PEV = PEV/PEV(end);    
%    Options.rank = nnz(lambda);
%    Options.nPC = min(find(PEV >= Options.PEV));
%    T = T(:,1:Options.nPC).*repmat(sqrt(lambda(1:Options.nPC)),[N, 1]);
%    P = (X'*T)./repmat(lambda(1:Options.nPC),[d, 1]);
%  end
%end  

end


return;
