function clustlist = pcaclust(varargin)
%
% PCA Clustering
%
%   [clust,f_idx] = pcaclust(X)
%   [clust,f_idx] = pcaclust(X, Options)
%   [clust,f_idx] = pcaclust(X, Y)
%   [clust,f_idx] = pcaclust(X, Y, Options)
% 
% X         [N -by- d_X]  data matrix, N samples, d_X variables 
% Y         [N -by- d_Y]  (soft) labels matrix, N samples, d_Y variables 
%           if Y is not supplied then unsupervised clustering will be applied
% Options.
%   lambda      supervision  level; ==0 -- no supervision, == 1 -- full supervision; 1 by default
%   perc        percentage of variables which will be shaved at each step; 5 by dehault
%   nclust      the maximal number of clusters to discover, routine can return less clusters
%               then nclust (e.g. if the rank of X is less then nclust); 5 by default
%   sclust      sizes of clusters, scalar or [1- by -nclust] row; 
%               nan means auto selection; nan by default  

%   X_centering do nothing (==[] or 0), do mean centering (==nan), center around some vaue v (==v); by default  =[]
%   Y_centering do nothing (==[] or 0), do mean centering (==nan), center around some vaue v (==v); by default  =[]
%   X_scaling   do nothing (==[] or 1), divide each col by std (==nan), divide by some v (==v); by default =[]
%               selecting X_scaling == nan is equivalent to target == 'cor'   
%   Y_scaling   do nothing (=[] or 1), divide each col by std (=nan), divide by some v (=v); by default =[]
%               Y_scaling does not influence result and was added due to symmetry/compatibility reasons
%
%   target      maximization target: 'cov' for covariation, 'cor' for correlation, 
%               if 'cor' selected then AFTER all preprocessings columns of X will be divided 
%               by their lengthes; 'cov' by default
%             
%
%   output    ~=0 enables control output, ==0 disables it; 0 by default 
%
%
% 
% clustlist.    (structure array) 
%   size    the number of vars in the clust
%   varind  indices of the vars in the cluster
%   varsgn  signs of the vars (+1/-1 means that var is correlated/anticorrelated with the first PC)
%   sgnmean signed mean over vars of the cluster 
%   corPC   (abs) correlations between the var and the first PC
%   corSM   (abs) correlations between the var and smean
%   V_b     variance between vars
%   V_w     variance within vars
% 
%

% Copyright: S.Verzakov, serguei@ph.tn.tudelft.nl
% Faculty of Applied Sciences, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

DefaultOptions.lambda = 1;
DefaultOptions.perc = 5;
DefaultOptions.nclust = 5;
DefaultOptions.sclust = nan;
DefaultOptions.X_centering = [];
DefaultOptions.Y_centering = [];
DefaultOptions.X_scaling = [];
DefaultOptions.Y_scaling = [];
DefaultOptions.target = 'cov';
DefaultOptions.output = 0;

[X, Y, Options] = CheckInput(DefaultOptions, varargin{:});

[N, d_X] = size(X);
[X, Options.X_centering, Options.X_scaling] = prepro(X, Options.X_centering, Options.X_scaling);

if strcmpi(Options.target,'cor')
  X = X ./repmat(sqrt(sum(X.*X,1)), [size(X,1) 1]);  
  %X = X ./repmat(std(X,1), [size(X,1) 1]);
  %X = X ./repmat(std(X,0,1), [N 1]);
end

if ~isempty(Y) 
  [Y, Options.Y_centering, Options.Y_scaling] = prepro(Y, Options.Y_centering, []);
  Q = orth(Y);
  %Q = Y./repmat(sqrt(sum(Y.*Y,1)), [N 1]);
  %if Options.lambda < 1;
  %R = null(Q');
  %Q = [Q, sqrt(1-Options.lambda)*R];
  %end  
else
  Q = [];
end


clustlist.size    = [];
clustlist.varind  = [];
clustlist.varsgn  = [];
clustlist.sgnmean = [];
clustlist.corPC   = [];
clustlist.corSM   = [];
clustlist.V_b     = [];
clustlist.V_w     = [];
clustlist         = clustlist(2:1);

X0 = X; 
varind = [1:d_X];

for c=1:Options.nclust
  if Options.output==1
    disp(['cluster #',num2str(c)]);
  end

  local_clustlist = Shave(X, Options.perc, Q, Options.lambda, Options.output, c);
  
  if isempty(local_clustlist)
    break;
  end


  clust_size = [local_clustlist(:).size];

  if isnan(Options.sclust(c))
    X_rnd = X;
    for j=1:d_X
      X_rnd(:,j) = X_rnd(randperm(N)',j);
    end
 
    if Options.output==1
      disp(['cluster #',num2str(c),' (randomized)']);
    end

    local_clustlist_rnd = Shave(X_rnd, Options.perc, Q, Options.lambda, Options.output, c);  
    clust_size_rnd = [local_clustlist_rnd(:).size];

    if ~isequal(length(clust_size),length(clust_size_rnd))
      error('bug: cluster sizes are not equal for original and randomized datasets');
    end
    
    R = [local_clustlist(:).V_b]./([local_clustlist(:).V_b] + [local_clustlist(:).V_w]);
    R_rnd = [local_clustlist_rnd(:).V_b]./([local_clustlist_rnd(:).V_b] + [local_clustlist_rnd(:).V_w]);

    [max_gap,ind] = max(R-R_rnd);

  else
    ind = find(clust_size <= Options.sclust(c));
    ind = ind(1);
  end

  if Options.output==1
    disp(['size of cluster #',num2str(c),': ' num2str(local_clustlist(ind).size)]);
  end
  
  clustlist(c) = local_clustlist(ind);

  local_varind = clustlist(c).varind;
  clustlist(c).varind = varind(local_varind);

  sgnX0 = X0(:,clustlist(c).varind).*repmat(clustlist(c).varsgn,[N 1]);
  clustlist(c).sgnmean = mean(sgnX0, 2);
  clustlist(c).corSM = clustlist(c).sgnmean' * sgnX0;
  clustlist(c).corSM = clustlist(c).corSM./sqrt(sum(clustlist(c).sgnmean.^2,1) * sum(sgnX0.^2,1));
  clustlist(c).V_b = var(clustlist(c).sgnmean,1);
  clustlist(c).V_w = mean(var(sgnX0',1)',1);

  varind(local_varind) = [];
  d_X = length(varind);
  if isempty(varind) 
    break;
  end
  
  sgnmeanX = mean(X(:,local_varind).*repmat(clustlist(c).varsgn,[N 1]),2);
  X(:,local_varind) = [];
  %X = X - (clustlist(c).sgnmean*(clustlist(c).sgnmean'*X))/(clustlist(c).sgnmean'*clustlist(c).sgnmean); 
  X = X - (sgnmeanX*(sgnmeanX'*X))/(sgnmeanX'*sgnmeanX); 
end

return;


%------------------------------------------------------- Shave ---
function clustlist = Shave(X,perc,Q,lambda,output,cur_clust_num)
[N,d] = size(X);

X0 = X;
N0 = N;
d0 = d;
varind = [1:d];

if ~isempty(Q)
  if lambda == 1
    X = Q'*X;
    N = size(Q,2);
  
  else  
    X = (1-lambda)*X + lambda*Q*(Q'*X);
  end
end

clustlist.size    = [];
clustlist.varind  = [];
clustlist.varsgn  = [];
clustlist.sgnmean = [];
clustlist.corPC   = [];
clustlist.corSM   = [];
clustlist.V_b     = [];
clustlist.V_w     = [];
clustlist = clustlist(2:1);

c = 1;
while 1
  if length(varind)==1
    break
  end

  if N >= d
    [v,lambda,v] = svds(X'*X,1);
    lambda = sqrt(lambda);
    v = sqrt(lambda)*v';
  else
    [u,lambda,u] = svds(X*X',1);
    v = u'*X;
  end

  varsgn = sign(v);
  varsgn(varsgn == 0) = 1;

  v = abs(v);
  corPC = v./sqrt(sum(X.^2,1));
  
  [tmp, sortind] = sort(v);
  ind = sortind(ceil(length(sortind)*perc/100)+1:end);
  if isempty(ind)
    ind = sortind(end);
  end

  d = length(ind);
  X = X(:,ind);     
  varind = varind(ind);
  varsgn = varsgn(ind);
  corPC  = corPC(ind);

  clustlist(c).size    = d;
  clustlist(c).varind  = varind;
  clustlist(c).varsgn  = varsgn;
  clustlist(c).sgnmean = [];
  clustlist(c).corPC   = corPC;
  clustlist(c).corSM   = [];
  
  sgnX = X0(:,varind).*repmat(varsgn,[N0 1]);
  %sgnX = X.*repmat(varsgn,[N 1]);
  clustlist(c).V_b = var(mean(sgnX,2),1);
  clustlist(c).V_w = mean(var(sgnX',1)',1);

  if output==1
    disp(['Looking for the cluster ', num2str(cur_clust_num),'; step ', num2str(c), '; current cluster size: ', num2str(length(varind))]);
  end

  c = c+1;
end

return;


%-------------------------------------------------- CheckInput ---
function [X, Y, Options] = CheckInput(DefaultOptions, varargin)

switch nargin
case 2
  X = varargin{1};
  Y = [];
  Options = [];

case 3
  X = varargin{1};
  if isa(varargin{2}, 'struct')  
    Y = [];
    Options = varargin{2};
  else  
    Y = varargin{2};
    Options = [];
  end

case 4
  X = varargin{1};
  Y = varargin{2};
  Options = varargin{3};

otherwise
  error('Invalid number of the input arguments');
end

Options = updstruct(DefaultOptions, Options);

if isempty(X)
  error('Input matrix X is empty');
end

[N_X, d_X] = size(X);

if ~isempty(Y) & Options.lambda ~= 0
  [N_Y,d_Y] = size(Y);
  if N_Y ~= N_X
    error('size(X,1) must be equal to size(Y,1)');  
  end
else
  Y = [];
  Options.lambda = 0;
end

N = N_X;

if Options.perc <= 0 | Options.perc >= 100
  error('Options.perc must be >0, <100');  
end

if floor(Options.nclust) ~= Options.nclust| Options.nclust < 1 
  error('Options.nclust must be integer, >=1');  
end

if length(Options.sclust(:)) ~= 1 & length(Options.sclust(:)) ~= Options.nclust
  error('length(Options.nclust) must be =1 or =Options.nclust');  
end

if any((floor(Options.sclust) ~= Options.sclust | Options.sclust < 1) & ~isnan(Options.sclust)) 
  error('Options.sclust must be integer, >=1 or nan');  
end

if length(Options.sclust) == 1;
  Options.sclust = repmat(Options.sclust,[1, Options.nclust]);
else
  Options.sclust = Options.sclust(:).';
end


if ~strcmpi(Options.target,'cov') & ~strcmpi(Options.target,'cor')
  error('Options.target must be ''cov'' or ''cor''');  
end

if length(Options.output) ~= 1 
  error('Options.output must be scalar');  
end


return



