% SEGMENT_COMB Segmentation combining spatial and spectral domains
%
%    [SEG,ALLSEG]=SEGMENT_COMB(A,CC,WSPEC,SIGMA,MAXIT)
%
% INPUT
%   A       dataset with multi-band image data
%   CC      requested number of classes (may be read from labeled A)
%   WPEC    untrained spectral classifier (optional, default: nmc)
%   SIGMA   smoothing parameter (optional, default: 1.0)
%   MAXIT   maximum number of iterations (optional, default: 20)
%
% OUTPUT
%   SEG     segmentation result as a dip_image
%   ALLSEG  all steps of the segmentation process
%
% DESCRIPTION
%
% Multi-band image is segmented using the combined classifier segmentation
% algorithm. Initial labeling is needed. If it is not supplied in the input
% dataset, it is estimated via kmeans algorithm. Then, spectral (WSPEC) and
% spatial (Parzen classifierimplemented as convolution with Gaussian kernel
% and smoothing SIGMA). The process is iterated until a stable solution is
% found or a maximum number of iterations is reached.
%
% $Id: segment_comb.m,v 1.4 2005/02/10 15:24:07 pavel Exp $

% Copyright: Pavel Paclik, pavel@ph.tn.tudelft.nl
% Faculty of Applied Physics, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

function [seg,allseg]=segment_comb(a,cc,wspec,sigma,maxit)
 
	[sc,fc,tcc]=getsize(a);

	if ~exist('cc','var') 
		error('number of classes must be specified!');
	end
	
	if tcc<=1
		[lab,b]=kmeans(a,cc);
	else
		prwarning(2,'using labels supplied in the dataset');
		b=a;
		cc=tcc;
	end

	if ~exist('wspec','var')
		prwarning(2,'spectral classifier not specified, using wspec=nmc');
		wspec=nmc;
	end
	
	if ~exist('sigma','var')
		prwarning(2,'smoothing sigma not specified, using sigma=1.0');
		sigma=1;
	end
	
	if ~exist('maxit','var')
		maxit=20;
	end
	
	b.objsize=a.objsize;
   
	seg=b.nlab;
	allseg=seg;
	
	lastseg=zeros(size(seg));
	
	it=1;
	err=sum(sum( abs( seg-lastseg ) ));
	while it<=maxit
		
		fprintf('[%d',it);
		bspec=b*(b*wspec); % traing and applying the spectral classifier
		
		% spatial classifier
		bspat=spatm(bspec,sigma);
		bspat=bspat(:,cc+1:end);
		
		% combining
		if size(bspec)==size(bspat)

			b=bspec.*bspat;
			
			% new labelling
			lastseg=seg;
			seg=b*classd;
			
			allseg=[allseg seg];

			err=sum(sum( abs( seg-lastseg ) ));      
			
			fprintf(' err=%d] ',err);
			it=it+1;
		else
			prwarning(1,'maximum number of %d iterations reached, returning the last result', maxit);
			it=maxit+1;
		end
		
		% in case of stable classification, finish
		if err==0
			it=maxit+1;
		end
		
	end
	
	seg=getli(int8(seg),a);
	allseg=getli(int8(allseg),a);
   
   return