% SHOWSI Show spectral image
%
%   H=SHOWSI(DATA,FORMAT)
%
% INPUT
%    DATA     spectral image as a dip_image object
%    FORMAT   optional data format for conversion to dip_image
%             (def: 'double') for info, see dip_image
%
% OUTPUT
%    H        handle of the spectral visualization tool
%
% DESCRIPTION
% 1. Visualization mode (default)
% Input dataset or a dip_image with spectral data is visualized in two
% windows: 2D spatial image and spectral plot.  Moving over the spatial
% image, the spectrum at a given spot is rendered in the spectral plot.  By
% clicking, points may be emphasized in the spatial image. Their spectra
% get drawn in the spectral figure. Three different colors may be used to
% distinguish the points/spectra. The set of points may be retrieved as a
% dataset using the si_get_spectra command. The band, shown in the
% spatial image may be interactively changed by clicking and dragging a
% line in the spectral figure.
%
% 2. Area measurement mode
% Area mode, invoked from the Actions menu in the spatial image, allows the
% user to define line in the spectrum used for measuring the area.  First,
% the desired spectrum is selected by moving the mouse pointer over the
% spatial image. By clicking, the spectrum is fixed and the two points may
% be selected in the spectral figure forming a line. An image is generated
% with color reflecting the area between the spectrum and the line.  Right
% mouse button cancels the last step.
%
% Additional images depicting the same object may be attached to the
% specral image. Dragging the mouse over the image shows corresponding
% spectrum from the spectral image.
%
% SEE ALSO
% SI_ATTACH_DISPLAY, SI_DETTACH_DISPLAY, SI_GET_SPECTRA, SI_SET_YLIM

% $Id: showsi.m,v 1.18 2007/03/01 14:11:07 serguei Exp $

% Copyright: Pavel Paclik, pavel@ph.tn.tudelft.nl
% Faculty of Applied Physics, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

function h=showsi(data,format)

	if isa(data,'dataset')
		%XXX
    % copy of all the user's data
	  user=data.user;
		
    if ~isempty(user) & isfield(user, 'type') & strcmp(user.type,'spectra') ...
    %& isfield(data.user, 'format') & (strcmp(data.user.format,'FTIR converted from bio-rad') | strcmp(data.user.format,'FTIR converted from Varian'))
      if isfield(user,'start_wavelength') & ~isempty(user.start_wavelength) & isfield(user,'step') & ~isempty(user.step)
			% set of bands
			  bands=user.start_wavelength:user.step:user.start_wavelength+(size(data,2)-1)*user.step;
      else
        bands = data.featlab;
        if ischar(bands)
          bands = str2num(bands);
          if isempty(bands)
            clear bands;  
          end
        end
      end
      
      % band axis units
			if isfield(user,'units') & ~isempty(user.units)
        units=user.units;
      else
        units = '';
      end
    end

    if ~exist('format','var'), format='double'; end
		data=data2dip(data,format);
	end
	
	if ~isa(data,'dip_image')
		error('dip_image required')
	end
	
	if length(size(data))~=3
		error('spectral image (3D array) is required');
	end
	
	% generate default set of bands?
	if ~exist('bands','var')
		bands=1:size(data,3);
		units='1';
		user=[];
	end
	
	% primary image (pim): spectral image as a dipimage
	hpim=dipshow(data,'percentile','notruesize');
	udpim=get(hpim,'UserData');
	udpim.si_type='primary image'; % this is the primary image display
	updim.si_ax=gca;
	
	% spec: spectral domain figure
	hspec=figure;
	%XXX
  t = get(hspec,'position');
  t([2,4]) = t([2,4])+[-0.1,+0.1]*t(4);
  set(hspec,'position',t);

  udspec=get(hspec,'UserData');	
	udspec.si_type='spectral plot'; % this is the spectral plot
	udspec.si_ax=gca;
	set(udspec.si_ax,'units','normalized');
  %XXX
  t = get(udspec.si_ax,'position');  
  t([2,4]) = t([2,4])+[0.1,-0.1]*t(4);;
  set(udspec.si_ax,'position',t);

	% link from one objects to another
	udpim.hspec=hspec;
	udpim.hpim=hpim; % handle to itself
	udspec.hpim=hpim;
	udspec.hspec=hspec; % handle to itself

	udspec.si_bands=bands; % set of spectral bands
	udpim.si_bands=bands; % used often -> stored in both
	
	udspec.si_units=units;
	udpim.si_points=[]; % coordinates of selected points ( [x,y,label] )
	udpim.si_user=user; % user data defining spectral dataset

	udpim.si_attached_displays=[]; % handle to attached displays
	udpim.si_attached_scatters=[]; % handle to attached scatterd
	
	set(hspec,'busyaction','cancel','DoubleBuffer','on')

	% plot the spectral curve which will be used for fast updates
	udspec.si_templine=plot(bands,zeros(size(data,3),1),'-','color',[0.5 0.5 0.5]);
	if strcmp(units,'cm^-1')
		set(udspec.si_ax,'XDir','reverse')
	end
	hold on

	set(udspec.si_ax,'xlim',[min(bands) max(bands)])
	t=max(max(data));
	set(udspec.si_ax,'ylim',[0 t])
	
	udspec.si_cursliceline=plot([bands(udpim.curslice+1) bands(udpim.curslice+1)],...
				    [0 t],...
				    '-','color',[1 0 0]);
	udspec.si_movement=0; % the band movement is switched off in
			      % the beginning
	set(hspec,'WindowButtonDownFcn','showsi_update(''down_spec'')');	

	% radion buttons for classes
	
	% class color palette 
	%udspec.si_classcolmap=[... 	
  %       0         0    1.0000; ...
  %       0    0.5000         0; ...
  %  1.0000         0         0; ...
  %       0    0.7500    0.7500; ...
  %  0.7500         0    0.7500; ...
  %  0.7500    0.7500         0; ...
  %  0.2500    0.2500    0.2500];
 	%udspec.si_classcolnam={'blue'; 'green';'red';'cyan';'magenta';'yellow';'black'};
	
  udspec.si_classcolmap=get_spec_colormap;
 	% remove white
  udspec.si_classcolmap(all(udspec.si_classcolmap == 1,2), :) = [];
	% put black at the end
  udspec.si_classcolmap(all(udspec.si_classcolmap == 0,2), :) = [];
  udspec.si_classcolmap(end+1,:) = [0 0 0];
  udspec.si_classcolnam=num2cell([repmat('class',[size(udspec.si_classcolmap,1) 1]), ...
                        num2str((1:size(udspec.si_classcolmap,1))')], 2);
	
  udspec.si_class=1;
  udpim.si_classcolmap=udspec.si_classcolmap; % colormap is stored in both pim and spec


  % area mode: points in the spec figure (fast update)
	udspec.si_area_line=plot([0 0],[0 0]);

	set(udspec.si_area_line,'visible','off');
	set(udspec.si_area_line, 'color', [0 0 1],'marker','.');

	udspec.si_lst=uicontrol('style','listbox','value',1,...
			       'string',udspec.si_classcolnam,...
			       'units','normalized',...
             'ButtonDownFcn','',...
			       'callback','showsi_update(''lst'')');
	%keyboard
  t=get(udspec.si_ax,'position');
	set(udspec.si_lst,'position',[t(1) 0.02 0.2 0.12]);
	
  plotcolor = udspec.si_classcolmap(udspec.si_class,:);
  weights = [1; 1.5; 0.5]; 
  if sum(plotcolor*weights/sum(weights)) >= 0.5
    textcolor = [0 0 0];
  else
   textcolor = [1 1 1];
  end
  udspec.si_colindicator=uicontrol('style','text','backgroundcolor',udspec.si_classcolmap(1,:),...
                                   'foregroundcolor',plotcolor,'string',{'class', 'color'}, ...
                                   'fontweight', 'normal', 'fontangle','normal',...
                                   'units','normalized','position',[t(1)-0.08 0.02 0.07 0.06]);
  
  
  %set(udspec.si_lst,'position',[t(1) 0.02 0.2 0.05]);
  
  
  
  
  if 0
	% button 1
	udspec.si_rb1=uicontrol('style','radio','value',1,...
			       'string','blue',...
			       'units','normalized',...
				'ButtonDownFcn','',...
			       'callback','showsi_update(''rb'',1)');
	t=get(udspec.si_rb1,'extent');
	set(udspec.si_rb1,'position',[0.1 0.02 1.5*t(3) 1.01*t(4)]);

	% button 2
	udspec.si_rb2=uicontrol('style','radio','value',0,...
			       'string','red',...
			       'units','normalized',...
			       'callback','showsi_update(''rb'',2)');
	t2=get(udspec.si_rb2,'extent');
	set(udspec.si_rb2,'position',[0.25 0.02 1.5*t2(3) 1.01*t2(4)]);

	% button 3
	udspec.si_rb3=uicontrol('style','radio','value',0,...
			       'string','green',...
			       'units','normalized',...
			       'callback','showsi_update(''rb'',3)');
	t=get(udspec.si_rb3,'extent');
	set(udspec.si_rb3,'position',[0.4 0.02 1.5*t(3) 1.01*t(4)]);
	end
	
	% update window titles
	set(hpim,'Name',sprintf('band: %f', bands(udpim.curslice+1)));
	set(hspec,'Name',sprintf(' band: %f ',bands(udpim.curslice+1)));
	
	set(hpim,'WindowButtonDownFcn','showsi_update(''down_dip'')');
	set(hpim,'WindowButtonMotionFcn','showsi_update(''motion_dip'')');	

	% closing one window must close the other one:
	set(hpim,'CloseRequestFcn','showsi_update(''close'')');
	set(hspec,'CloseRequestFcn','showsi_update(''close'')');

	% store the handles in both plots
	set(hpim,'UserData',[]);
	set(hpim,'UserData',udpim);
	
	set(hspec,'UserData',[]);
	set(hspec,'UserData',udspec);
	
	% adjust the pim figure menu bar
	h = findobj(hpim,'Type','uimenu','tag','actions');
	% visualization mode selector
	uimenu(h,'Label','spectra visualisation','Tag', ...
	       'mouse_si_visualisation','Callback','si_setvismode(gcbf);');
	% put the menu item to the first position:
	h2=get(h,'children');
	set(h,'children',[h2(2:end);h2(1)]);
	si_setvismode(hpim);	

	% area mode
	uimenu(h,'Label','spectra area','Tag', ...
	       'mouse_si_area','Callback','si_setareamode(gcbf);');
	% put the menu item to the first position:
	h2=get(h,'children');
	set(h,'children',[h2(2:end-1);h2(1);h2(end)]);
	
	
	h=hpim;
	
	return
