%simca_apply SIMCA (applying)
%  
%  [labels, is_outl, Q_red, T2_red] = simca_apply(X,SIMCAModel)
%
% INPUT 
%   X             [N -by- d] data matrix (or dataset), N samples, d variables
%   SIMCAModel    structure array returned by simca_train
%
% OUTPUT
%   labels          classification (without outlier detection)       
%   is_outl         outlier detection
%   Q_red, T2_red  [N -by- number_of_classes]  statistics reduced by critical levels
%
%
% DESCRIPTION
% Similarity (distance) matrix = sqrt(Q_red.^2 + T2_red.^2)
% Sample is outlier if (Q_red > 1) or (T2_red > 1)
%
% SEE ALSO
% simca_train

% Copyright: S. Verzakov, serguei@ph.tn.tudelft.nl
% Faculty of Applied Sciences, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

% $Id: simca_apply.m,v 1.8 2004/07/19 11:23:41 pavel Exp $

function [labels, is_outl, Q_red, T2_red] = simca_apply(X,SIMCAModel)

X = +X;

[N, d] = size(X);
number_of_classes = length(SIMCAModel);
Q_red = zeros(N,number_of_classes);
T2_red = zeros(N,number_of_classes);


if exist('prwarning')
  global PRWARNING
  if isempty(PRWARNING)
    prwarning;
  end  
  wl = PRWARNING;
  prwarn = 1; 
else
  wl = warning;
  prwarn = 0; 
end

for c=1:number_of_classes
  T = pca_apply(X,SIMCAModel(c).P,SIMCAModel(c).Options);

  if prwarn 
    prwarning(1);
  else
    warning('off');
  end 

  Stat = pca_stat(X, T, SIMCAModel(c).P, SIMCAModel(c).lambda, ...
    SIMCAModel(c).Options.nPC, SIMCAModel(c).Options);

 if prwarn 
   prwarning(wl);
 else
   warning(wl);
 end 

 if SIMCAModel(c).Crit.Q==0
   Q_red(:,c) = 0;

   WarnMsg = 'the model space is equal to the training data set space, Q distance will be set to zero';
   if prwarn
     prwarning(2,WarnMsg);
   elseif Options.verbose
    disp(['simca_apply: ' WarnMsg]);
   end  

  else
    Q_red(:,c) = Stat.Q ./ SIMCAModel(c).Crit.Q;
  end

  T2_red(:,c) = Stat.T2 ./ SIMCAModel(c).Crit.T2; 
end

[d_2, labels] = min(Q_red.^2 + T2_red.^2,[],2);
ind = sub2ind([N number_of_classes], [1:N]', labels);
is_outl = Q_red(ind) > 1 | T2_red(ind) > 1;

return;
