%simca_train SIMCA (training)
%  
%  SIMCAModel = simca_train(X,labels)
%  SIMCAModel = simca_train(X,labels,Options)
%  SIMCAModel = simca_train(D)
%  SIMCAModel = simca_train(D,Options)
%
% INPUT 
%  X             [N -by- d] training input data matrix, N samples, d variables
%  labels        [N -by- 1] labels(i) is the number of class to which i-th sample belongs;
%                lables should be integer numbers in the region [1; number_of_classes]
%  D             labeled dataset (PRTools)  
%
%  Options.
%   nPC          [1 -by- number_of_classes] 
%                nPC(j)==nan means autodetection; by default nan
%   nPC_em       nPC estimation method: 'PEV', 'MI', 'F', 'KG', 'PRESS'; by default 'PEV'     
%   PEV          part of explained varince; if nPC(j)==nan then the number of principal components to return
%                is the minimum number of PC such that the portion of explained variance is >= PEV(j); 
%                by default 0.95
%                if mean centering was not performed then PEV(j) defines the level 
%                not for the explained variance but for the explained sum of squares; 
%   F_cl         confidence level  for Mlinowski F-test  
%   conf_level   confidence level for crit values of Q and T2 stats; by default 0.95
%   use_ecv      force the usage of empirical critical values; by default 0
%
% OUTPUT
%  SIMCAModel       
%
%
%
% DESCRIPTION
%  
%
% SEE ALSO
% pca_train

% Copyright: S. Verzakov, serguei@ph.tn.tudelft.nl
% Faculty of Applied Sciences, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

% $Id: simca_train.m,v 1.5 2005/05/05 14:35:55 serguei Exp $

function SIMCAModel = simca_train(varargin)

Options = [];

if isa(varargin{1},'dataset')
 X = +varargin{1};
 labels = varargin{1}.nlab;
 X(labels == 0,:) = [];
 labels(labels == 0) = [];
  if nargin > 1
   Options = varargin{2}; 
 end
else
 X = varargin{1};
 labels = varargin{2};
  if nargin > 2
   Options = varargin{3}; 
 end
end
clear varargin;

[N, d] = size(X);

DefaultOptions.nPC = nan;
DefaultOptions.nPC_em = 'PEV';
DefaultOptions.PEV = 0.95;
DefaultOptions.F_cl = 0.95;
DefaultOptions.conf_level = 0.95;
DefaultOptions.use_ecv = 0;
Options = updstruct(DefaultOptions, Options);

number_of_classes = max(labels);

if length(Options.nPC) == 1
  Options.nPC = repmat(Options.nPC, [1 number_of_classes]);
end

if length(Options.PEV) == 1
  Options.PEV = repmat(Options.PEV, [1 number_of_classes]);
end

if length(Options.conf_level) == 1
  Options.conf_level = repmat(Options.conf_level, [1 number_of_classes]);
end

if length(Options.F_cl) == 1
  Options.F_cl = repmat(Options.F_cl, [1 number_of_classes]);
end

if exist('prwarning')
  global PRWARNING
  if isempty(PRWARNING)
    prwarning;
  end  
  wl = PRWARNING;
  prwarn = 1; 
else
  wl = warning;
  prwarn = 0; 
end

for c=1:number_of_classes
  X_c = X(labels==c,:);
  SIMCAModel(c).Options.M = size(X_c,1)-1;
  SIMCAModel(c).Options.nPC = Options.nPC(c);
  SIMCAModel(c).Options.nPC_em = Options.nPC_em;  
  SIMCAModel(c).Options.PEV = Options.PEV(c);
  SIMCAModel(c).Options.F_cl = Options.F_cl(c);
  SIMCAModel(c).Options.X_centering = nan;  
  SIMCAModel(c).Options.X_scaling = [];  
  
  %Opt = SIMCAModel(c).Options;
  %Opt.nPC = nan;
  %Opt.nPC_em = 'ALL';
  %Opt.nPC_em = 'NOPRESS';
  %Out = pca_npc(X_c,Opt);  
  %Out
  %keyboard

  [T,SIMCAModel(c).P,SIMCAModel(c).lambda,SIMCAModel(c).Options] = pca_train(X_c,SIMCAModel(c).Options);
  
  if prwarn 
    prwarning(1);
  else
    warning('off');
  end 

  Stat = pca_stat(X_c, T, SIMCAModel(c).P, SIMCAModel(c).lambda, ...
    SIMCAModel(c).Options.nPC, SIMCAModel(c).Options);

  Crit = pca_crit(Options.conf_level(c), SIMCAModel(c).lambda, ...
    SIMCAModel(c).Options.nPC, SIMCAModel(c).Options, Stat, Options.use_ecv);
  
  if prwarn 
    prwarning(wl);
  else
    warning(wl);
  end 

  if Options.use_ecv
    SIMCAModel(c).Crit.Q = Crit.Q_emp;    
    SIMCAModel(c).Crit.T2 = Crit.T2_emp;    
  else
    SIMCAModel(c).Crit.Q = Crit.Q;    
    if isnan(Crit.T2)
      SIMCAModel(c).Crit.T2 = Crit.T2_emp;    
    else
      SIMCAModel(c).Crit.T2 = Crit.T2;    
    end
  end
end

return;
