% SIMPLECLUST simple and robust (and heuristic) clustering
%
%    [W,B,MODES]=SIMPLECLUST(A,WDIST,PERC)
%
% INPUT
%   A - dataset (may be a single class)
%   WDIST - dissimilarity mapping (def: dissimm('dasam'))
%   PERC - percentile of cluster counts (def: 0.5 == median)
%
% OUTPUT
%   W   - trained classifier capable of labeling new dataset
%   B   - labeled input dataset A
%   MODES - dataset with data modes samples
%
% DESCRIPTION
% This function implements a simple & dirty clustering using modeseek
% algorithm. First, outliers are detected and removed using PCA model
% (SIMCA).  MODESEEK algorithm with different neighborhood sizes is used to
% find resulting numbers of clusters. One-cluster solution is neglected.
% Percentile (by default median) of number of clusters is taken as the most
% viable solution.  Mapping capable of labeling new data, original
% labeled dataset and identified modes are returned.

% $Id: simpleclust.m,v 1.2 2005/10/19 11:33:37 serguei Exp $

function [w,b,modes,cc]=simpleclust(a,wdist,perc)

  if ~exist('wdist','var') | isempty(wdist), wdist=dissimm('dasam'); end
  if ~exist('perc','var'), perc=0.5; end    
  if ~isdataset(a), a=dataset(a); end
    
    % detect outliers
    % sm=simca_train(a);
    % [lab,isout]=simca_apply(a,sm);
    
    % get dataset without outliers
    % b=a(find(isout==0),:);
    b=a;

    % construct a dissimilarity
    % wdist=b*wdist;
    % D=b*wdist;
    D = b*(b*wdist);

    % clear wdist % save memory
    
    cc=[]; % space for number of clusters found by modeseek
    finish=0;
    k=2; % we start from neighborhood of 2
    seen_ones=0; % how many ones we found (for stopping crit)
    
    while finish==0
      [lab,J]=modeseek(D,k);
      
      cc=[cc length(J)];
      
      if length(J)==1, seen_ones=seen_ones+1; end
      
      k=k+1;
      
      % we stop if we passed all neighborhood sizes or
      % if we already saw 5 ones (all data form a single cluster 
      % at this level)
      
      if k==size(D,1) | seen_ones>5, finish=1; end

      % fprintf(1,'.');
    end
	
    % retrieve 'reasonable' number of clusters
    cc=cc(find(cc>1)); % remove ones

    c=get_percentile(cc,perc);
    
    % get the closest number of clusters (percentile may be float)
    [s,ind]=sort(abs(c-cc));
    k=ind(1)+2;

    [lab,J]=modeseek(D,k);

    % create a dataset with modes and class labels (in order to build
    % knnc on modes)
    modes=dataset(+b(J,:),genlab(ones(1,length(J))));
    fprintf(1,' %d ',size(modes,1));

    % untrained dissimilarity mapping
    %w=dissimm([],wdist{1},wdist{2});
    %w=modes*w; % train the dissimilarity measure
    w=modes*wdist;
    
    w2=(modes*w)*mindistc; % trained mindist classifier
    w=w*w2; % trained classifier

    % re-build modes as a dataset with original prototype class labels
    % Modes preserve their original labels.
    modes=b(J,:);

    if size(w,2)==1
      % we cannot invoke labeld as it assumes 1D dataset is a result
      % of a two-class discriminant :-)
      b=setlabels(a,ones(size(a,1),1));
    else
      b=setlabels(a,a*w*labeld); % apply it to the original dataset
    end

return

