%SIMPLISMA simple-to-use interactive self-modeling mixture analysis:
%          selects set of "pure" variables
%
%  [Z, SEL_VAR_IND, SEL_VAR_PURITY, PURITY, R_RATIO, ERR] = SIMPLISMA(X, OPT)
%
% INPUT 
%   X             [M -by- K] double matrix  
%
%   OPT.   
%    MAX_NUM      the max number of variables to select, INF means selecting as many variables as possible,
%                 until the rank of dataset is exausted; at least one variable will be always selected; by delault = INF
%    EPS          stopping criteion; algorithm stops ERR < EPS, such features are not included into the set; by default = 1e-2 
%    THETA        defines offset THETA*ABS(MAX(MAX(X))) used to supress noisy variables; by default = 1e-2 
%    VERBOSE      force/disable messages; if PRPROGRESS is available: PRPROGRESS(VERBOSE,['%s\n'],MSG) will be run, 
%                 if PRPROGRESS is not available: VERBOSE == [] or 0 -- no messages, VERBOSE ~= 0 runs DISP(MSG); 
%                 by default = []
%
%
% OUTPUT 
%   Z               [M -by- K_SEL] double matrix containing only selected variables
%   SEL_VAR_IND     [K_SEL -by- 1] vector of selected variables indices 
%   SEL_VAR_PURITY  [K_SEL -by- 2] two vectors of selected variables purities 
%                   in the first column are absolute purities, and in the second are relative purities
%   PURITY          [K_SEL -by- K -by- 2]  two matrices of purity spectra (purities of all variables at all selection steps)
%                   each row contains purities of all variables at particular selection step;
%                   the first matrix contains abs purities, the second one containes relative purities
%   R_RATIO         [K_SEL -by- 1] 
%   ERR             [K_SEL -by- 1] relative squared error of the data projection to the subspace spanned by the select features
%
% DESRIPTION
% Low level routine
% Implementation of the SIMPLISMA algorithm described in 
% "Interactive Self-Modeling Mixture Analysis" by W. Windig, J. Guilment, Anla. Chem. 119, 63, 1425-1432 
%
% Absolute dissim measure  abs_diss_(n+1) = ||dx||^2*det(Y_n^T Y_n)
% Relative dissim measure  rel_diss_(n+1) = ||dx||^2
%
% Here, Y_n is the set of n selected features (columns), which are normalized to have a unit length, dx is the part of 
% a feature x which is (part) orthogonal to span(Y). 
%
% SEE ALSO
% SIMPLISMAM, SIMPLISMAPS, OPA, VARIMAX, ALS

% Copyright: S.Verzakov, s.verzakov@ewi.tudelft.nl 
% Faculty EWI, Delft University of Technology
% P.O. Box 5031, 2600 GA Delft, The Netherlands

% $Id: simplisma.m,v 1.11 2006/03/15 12:17:15 serguei Exp $


function [Z, sel_var_ind, sel_var_purity, purity, R_ratio, err] = simplisma(X,Options)

if nargin < 2
  Options = [];
end

DefOptions.max_num = inf;
DefOptions.theta = 1e-2;
DefOptions.eps = 1e-2;
DefOptions.verbose = [];
Options = updstruct(DefOptions, Options);

%----------------
prprog = exist('prprogress');
if ~prprog & isempty(Options.verbose)
  Options.verbose = 0;
end
%----------------

[N, d] = size(X);
Options.max_num = min([Options.max_num, N, d]);

alpha = Options.theta*abs(max(X(:)));
X_ss = sum(X.^2,1);
SS = sum(X_ss);
X_m = mean(X,1);
X_std = sqrt(X_ss/N - X_m.^2);

norm_factor = (1/N)./(X_m.^2 + (X_std+alpha).^2);
R_factor = norm_factor.*X_std;
purity_factor = R_factor./(abs(X_m)+alpha);

R = sum(R_factor.*X_ss);
purity = purity_factor.*X_ss;
purity = cat(3,purity,purity);

[sel_var_purity, ind] = max(purity(1,:,2));
sel_var_purity = [sel_var_purity, sel_var_purity]; 
Det_factor = norm_factor(ind).*X_ss(ind);
Det = Det_factor;

O = X(:,ind)/sqrt(X_ss(ind));

Z = X(:,ind);
X(:,ind) = [];
X_ss(:,ind) = [];
dX_ss = X_ss;
norm_factor(ind) = [];
R_factor(ind) = [];
purity_factor(ind) = [];

sel_var_ind = ind;
var_ind = [1:d]';
var_ind(ind) = [];

dX_ss = dX_ss - ((O(:,end)'*X).^2);
err = sum(dX_ss)/SS;

R_new = sum(R_factor.*dX_ss);
R_ratio(1) = Det_factor*R_new/R;
R = R_new;

Msg = ['step 1; max purities are'...
         ' abs p(' num2str(sel_var_ind(1)) ')=' num2str(sel_var_purity(1,1)) ...
         ' rel p(' num2str(sel_var_ind(1)) ')=' num2str(sel_var_purity(1,2)) ...
         '; R_ratio=' num2str(R_ratio) '; err=' num2str(err)];

if prprog
    prprogress(Options.verbose,['%s\n'],Msg);
elseif Options.verbose
  disp(['simplisma: ' Msg]);
end  

for i=2:Options.max_num
  %if R_ratio < Options.eps
  if err(i-1) < Options.eps
    break;
  end

  purity(i,var_ind,2) = purity_factor.*dX_ss;

  [sel_var_purity(i,2), ind] = max(purity(i,var_ind,2));
  Det_factor = norm_factor(ind).*dX_ss(ind);
  
  purity(i,var_ind,1) = purity(i,var_ind,2)*Det;
  sel_var_purity(i,1) = sel_var_purity(i,2)*Det;
  Det = Det_factor*Det;
  
  O = [O, (X(:,ind) - O*(O'*X(:,ind)))/sqrt(dX_ss(ind))];
  
  Z = [Z, X(:,ind)];
  X(:,ind) = [];
  X_ss(:,ind) = [];
  dX_ss(:,ind) = [];  
  norm_factor(ind) = [];
  R_factor(ind) = [];
  purity_factor(ind) = [];

  sel_var_ind = [sel_var_ind; var_ind(ind)];
  var_ind(ind) = [];
  
  dX_ss = dX_ss - ((O(:,end)'*X).^2);
  err(i,1) = sum(dX_ss)/SS;
  
  R_new = sum(R_factor.*dX_ss);
  R_ratio(i,1) = Det_factor*R_new/R;
  R = R_new;
  
  Msg = ['step ' num2str(i) ...
         '; max purities are' ...
         ' abs p(' num2str(sel_var_ind(i)) ')=' num2str(sel_var_purity(i,1)) ...
         ' rel p(' num2str(sel_var_ind(i)) ')=' num2str(sel_var_purity(i,2)) ...
         '; R_ratio=' num2str(R_ratio(i)) '; err=' num2str(err(i))];

  if prprog
    prprogress(Options.verbose,['%s\n'],Msg);
  elseif Options.verbose
    disp(['simplisma: ' Msg]);
  end  
end

return


