%SIMPLISMAM simple-to-use interactive self-modeling mixture analysis:
%           selects set of "pure" variables
%  
%
%  W = SIMPLISMAM
%  W = SIMPLISMAM([],OPT)
%
%  W = A*W
%  [W, SEL_VAR_PURITY, PURITY, R_RATIO, ERR] = MAP(A, W)
%  [W, SEL_VAR_PURITY, PURITY, R_RATIO, ERR] = SIMPLISMAM(A, OPT)
%
%  C = B*W
%  C = MAP(B,W)
%  C = SIMPLISMAM(B,W)
%
%
% INPUT 
%   A             [M -by- K] training dataset
%   B             [N -by- K] test dataset
%
%   OPT.          optional SIMPLISMA settings
%    MAX_NUM      the max number of variables to select, INF means selecting as many variables as possible,a
%                 until the rank of dataset is exausted; at least one variable will be always selected; by delault = INF
%    EPS          stopping criteion; algorithm stops ERR < EPS, such features are not included into the set; by default = 1e-2 
%    THETA        defines offset THETA*ABS(MAX((MAX(A)))) used to supress noisy variables; by default = 1e-2 
%    VERBOSE      force/disable messages; if PRPROGRESS is available: PRPROGRESS(VERBOSE,['%s\n'],MSG) will be run; 
%                 by default = []
%
% OUTPUT 
%   W               [K -by- ?] untrained mapping or [K -by- K_SEL] trained feature selection mapping
%                   after training, indices of selected variables stored in W.DATA.IND  
%   SEL_VAR_IND     [K_SEL -by- 1] vector of selected variables indices 
%   SEL_VAR_PURITY  [K_SEL -by- 2] two vectors of selected variables purities 
%                   in the first column are absolute purities, and in the second are relative purities
%   PURITY          [K_SEL -by- K -by- 2] two matrices which rows are purity spectra; 
%                   each row contains purities of all variables at particular selection step
%                   the first matrix contains abs purities, the second one containes relative purities
%   R_RATIO         [K_SEL -by- 1] 
%   ERR             [K_SEL -by- 1] relative squared error of the data projection to the subspace spanned by the select features
%   C               [N -by- K_SEL] dataset, result of mapping execution on dataset B
%
% DESRIPTION
% Implementation of the SIMPLISMA algorithm described in 
% "Interactive Self-Modeling Mixture Analysis" by W. Windig, J. Guilment, Anla. Chem. 119, 63, 1425-1432 
%
% Absolute dissim measure  abs_diss_(n+1) = ||dx||^2*det(Y_n^T Y_n)
% Relative dissim measure  rel_diss_(n+1) = ||dx||^2
%
% Here, Y_n is the set of n selected features (columns), which are normalized to have a unit length, dx is the part of 
% a feature x which is (part) orthogonal to span(Y). 
%
% SEE ALSO
% OPAM, VARIMAXFM, SIMPLIMAPS, OPAPS, VARIMAXOM, ALS, SIMPLISMA

% Copyright: S.Verzakov, s.verzakov@ewi.tudelft.nl 
% Faculty EWI, Delft University of Technology
% P.O. Box 5031, 2600 GA Delft, The Netherlands

% $Id: simplismam.m,v 1.10 2007/01/31 21:52:19 serguei Exp $

function [w, sel_var_purity, purity, R_ratio, err] = simplismam(a,w)

% No dataset given: return untrained mapping.
if (nargin < 1) | (isempty(a))
  if nargin < 2
    w = [];
  end  
  w = mapping(mfilename,'untrained',{w});
  w = setname(w,'SIMPLISMA Mapping');
  return
end

isdataset(a);           % Assert that A is a dataset.

% training
if nargin < 2 | ~isa(w,'mapping') 
  if nargin < 2
    w = [];  
  end  

  [m,k] = getsize(a);
  
  [z, sel_var_ind, sel_var_purity, purity, R_ratio, err] = simplisma(+a,w);
  
  w_data.ind = sel_var_ind;

  %labels = [num2str([1:length(sel_var_ind)]','%-1d'),  num2str(sel_var_ind(:),'(%-1d)')];
  labels = a.featlab(sel_var_ind,:);

  % Save all useful data
  w = mapping(mfilename,'trained',w_data,labels,size(a,2),length(sel_var_ind));
  w = setname(w,'SIMPLISMA Feature Selection');
  
% applying
else 
  w_data = +w; % Unpack the mapping.
  %w = a*cmapm(w.size_in,w_data.ind);
  a = a(:,w_data.ind);
  %a.featlab = w.labels;
  units = specunits(a);
  a = remove_spectra_info(a);
  if units
    user = a.user;
    user.type = 'spectra';
    user.units = units;
    a.user = user;
  end  
  a.name = [a.name ': selected channels'];
  w = a;
end

return

