%SIMPLISMAPS simple-to-use interactive self-modeling mixture analysis:
%           selects set of "pure" objects (prototype selection)
%  
%
%  [B, SEL_OBJ_IND, SEL_OBJ_PURITY, PURITY, R_RATIO, ERR] = SIMPLISMAPS(A)
%  [B, SEL_OBJ_IND, SEL_OBJ_PURITY, PURITY, R_RATIO, ERR] = SIMPLISMAPS(A, OPT)
%
%
% INPUT 
%   A             [M -by- K] training dataset
%
%   OPT.   
%    MAX_NUM      the max number of objects to select, INF means selecting as many objects as possible,
%                 until the rank of dataset is exausted; at least one objects will be always selected; by delault = INF
%    EPS          stopping criteion; algorithm stops ERR < EPS, such objects are not included into the set; by default = 1e-2
%    THETA        defines offset THETA*ABS(MAX((MAX(A)))) used to supress noisy objects; by default = 1e-2 
%    VERBOSE      force/disable messages; if PRPROGRESS PRPROGRESS(VERBOSE,['%s\n'],MSG) will be run; 
%                 by default = []
%
%
% OUTPUT 
%   B               [M_SEL -by- K] dataset containing selected object (prototypes)
%   SEL_OBJ_IND     [M_SEL -by- 1] vector of selected objects indices 
%   SEL_OBJ_PURITY  [M_SEL -by- 2] two vectors of selected objects purities 
%                   in the first column are absolute purities, and in the second are relative purities
%   PURITY          [M -by- M_SEL -by- 2] two matrices which columns are purity profiles; 
%                   each columns contains purities of all objects at particular selection step;
%                   the first matrix contains abs purities, the second one containes relative purities
%   R_RATIO         [M_SEL -by- 1] 
%   ERR             [M_SEL -by- 1] relative squared error of the data projection to the subspace spanned by the select objects
%
% DESRIPTION
% Implementation of the SIMPLISMA algorithm described in 
% "Interactive Self-Modeling Mixture Analysis" by W. Windig, J. Guilment, Anla. Chem. 119, 63, 1425-1432 
% (applied to the transposed data matrix)
%
% Absolute dissim measure  abs_diss_(n+1) = ||dx||^2*det(Y_n Y_n^T)
% Relative dissim measure  rel_diss_(n+1) = ||dx||^2
%
% Here, Y_n is the set of n selected objects (rows), which are normalized to have a unit length, dx is the part of 
% an object x which is (part) orthogonal to span(Y). 
%
%
% SEE ALSO
% OPAPS, VARIMAXOM, SIMPLISMAM, OPAM, VARIMAXFM, ALS, SIMPLISMA

% Copyright: S.Verzakov, s.verzakov@ewi.tudelft.nl 
% Faculty EWI, Delft University of Technology
% P.O. Box 5031, 2600 GA Delft, The Netherlands

% $Id: simplismaps.m,v 1.6 2007/01/31 21:52:19 serguei Exp $

function [b, sel_obj_ind, sel_obj_purity, purity, R_ratio, err] = simplismam(a,Opt)

if nargin < 2
  Opt = [];
end

[y, sel_obj_ind, sel_obj_purity, purity, R_ratio, err] = simplisma((+a)',Opt);

purity = permute(purity,[2 1 3]);  
b = a(sel_obj_ind,:);
b.name = [b.name ': selected spectra'];

return

