% SMOOTHM Smoothing and derivatives mapping
%
%    W = SMOOTHM(NAME,SIGMA,DER)
%    W = SMOOTHM(NAME,WS,P,DER)
%
% INPUT
%   NAME   smoothing algorithm ('Gaussian' or 'SG')
%   SIGMA  sigma in Gaussian smoothing
%   WS     window size in Savitsky-Golay alg.
%   P      polynomial degree in SG
%   DER    derivative order (default: 0)
%
% OUTPUT
%   W      fixed smoothing mapping
%
% DESCRIPTION
% SMOOTHM is a fixed mapping capable of computing smoothed derivatives
% of spectra using either Savitzky-Golay or Gaussian smoothing.
% DIPImage is required when using Gaussian filtering.
%
% SEE ALSO
% DSPEC_SHAPE, DSHAPE, DERFILTER

% Copyright: Pavel Paclik, pavel@ph.tn.tudelft.nl
% Faculty of Applied Physics, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

% $Id: smoothm.m,v 1.5 2007/10/29 10:46:23 serguei Exp $

function W = smoothm(a,par1,par2,par3)

	if nargin == 0 
		W = mapping(mfilename,'fixed',{'Gaussian',1});
		W = setname(W,'Spectral smoothing mapping (Gaussian sigma=1.0)');	
	elseif ~isa(a,'dataset')
		% a is supposed to be smoothing type
		switch lower(a)
		 case {'gauss','gaussian'}
		  if ~exist('par1','var'), par1=1; end % sigma
		  if ~exist('par2','var'), par2=0; end % derivative
		  str=sprintf('sigma=%2.1f',par1);
		  if par2~=0
		      str=[str sprintf(',der=%1d',par2)];
		  end
		  W = mapping(mfilename,'fixed',{'Gaussian',par1,par2});
		  W = setname(W,...
				['Spectral smoothing mapping (Gaussian '...
				 str ')';]);
		 case {'sg','savitsky-golay'}
		  % polynom order
		  if ~exist('par1','var'), par1=3; end % window size
		  if ~exist('par2','var'), par2=1; end % polynom order
		  if ~exist('par3','var'), par3=0; end % derivative

		  % call sg to get coefficient matrix:
		  CM=sg(par1,par2,par3);

		  str=sprintf('ws=%d,p=%d,d=%d',par1,par2,par3);
		  W = mapping(mfilename,'fixed',{'Savitsky-Golay',CM});
		  W = setname(W,...
			      ['Spectral smoothing mapping (Savitsky-Golay ' ...
			       str ')']);
		 otherwise
		  error('unknown mapping type');     
		end
	elseif isa(a,'dataset')
		% execution 
		switch lower(par1)
		 case 'gaussian'
		  if ~exist('par3','var'), par3=0; end % derivative
		  b = (+a);
      if size(a,1) == 1
        b = [b; b];
      end  
      b = double(dip_gauss(b,[1 0],[par2 par2],[par3 0]));
      if size(a,1) == 1
        b = b(1,:);
      end  
      a = setdata(a,b);
      clear b;
      
		 case 'savitsky-golay'
		  % CM is in par2
		  a.data=sg(par2,+a,2); % in spectral domain
		 otherwise
		  error('unknown mapping type');
		end
		W=a;
	end


	return


