%VARIMAX Calculation of the new (sparse) loadings (axes) given the current
%
%  [P_MAX, V_MAX] = VARIMAX(P)
%  [P_MAX, V_MAX] = VARIMAX(P, OPT)
%
% INPUT 
%   P           [K -by- N] orthonormal loadings matrix (P'*P == EYE(N))
%               usually P columns are a few first (N) pca directions of K
%               dimensional dataset
%
%   OPT.   
%    N_ATTEMPTS   the number of independent attempts to maximize the varimax criterion; by default = 3
%    MAX_ITER     the max number of iterations wihin each attempt; each iteration involves 
%                 optimization over all unsorted pairs visited in random order; by default = 3 
%    SORT         put the simplest factors first; by default = 1;
%    EPS          convergence criterion (NEW_CRIT-OLD_CRIT < EPS*OLD_CRIT); by default = 1e-6
%    VERBOSE      force/disable messages; if PRPROGRESS is available: PRPROGRESS(VERBOSE,['%s\n'],MSG) will be run, 
%                 if PRPROGRESS is not available: VERBOSE == [] or 0 -- no messages, VERBOSE ~= 0 runs DISP(MSG); 
%                 by default = []
%
%
% OUTPUT 
%    P_MAX      [K -by- N] orthonormal loadings matrix (P_MAX'*P_MAX == EYE(N))
%    V_MAX      [1 -by- N] row of variances of squared factors (sparsity)
%
% DESCRIPTION
% This routine calculates what is called raw Kaiser varimax 
%
% SEE ALSO
% OPA, SIMPLISMA, ALS

% Copyright: S.Verzakov, s.verzakov@ewi.tudelft.nl 
% Faculty EWI, Delft University of Technology
% P.O. Box 5031, 2600 GA Delft, The Netherlands

% $Id: varimax.m,v 1.10 2007/01/19 00:39:19 serguei Exp $

function [P_max, v_max] = varimax(P, Options)

if nargin < 2
  Options = [];
end

DefOptions.n_attempts  = 3;
DefOptions.max_iter   = 3;
DefOptions.sort       = 1;
DefOptions.eps        = 1e-2;
DefOptions.verbose    = [];
Options = updstruct(DefOptions, Options);

%----------------
prprog = exist('prprogress');
if ~prprog & isempty(Options.verbose)
  Options.verbose = 0;
end
%----------------

P_0   = P;
P_max = P;

[k, n] = size(P);
v_max = var(P.*P,1);  
sum_v_max  = sum(v_max);
sum_v_0    = sum_v_max;


Msg = ['initial total var: ' num2str(sum_v_0)];
if prprog
    prprogress(Options.verbose,['%s\n'],Msg);
elseif Options.verbose
  disp(['varimax: ' Msg]);
end  

[i,j] = ind2sub([n,n],find(triu(ones(n),1)));
npairs = length(i);
ok = 0;
for a=1:Options.n_attempts
  sum_v = sum_v_0;
  for b=1:Options.max_iter
    ind = randperm(npairs);
    for c=1:npairs
      ii = i(ind(c));
      jj = j(ind(c));

      dp2 = diff(P(:,[jj, ii]).^2,1,2);
      pp = prod(P(:,[ii, jj]),2);

      x = k*(sum(dp2.^2) - 4*sum(pp.^2)) - sum(dp2).^2 - 4*sum(pp).^2;
      y = 4*(k*sum(dp2.*pp) - sum(dp2)*sum(pp));        
      r = sqrt(x.^2 + y.^2);

      cos4phi = x/r;
      cosphi2 = 0.5*(1 + sqrt(0.5*(1+cos4phi)));
      cosphi = sqrt(cosphi2);
      sinphi = sign(y)*sqrt(1 - cosphi2);
      %[cos(0.25*atan2(y,x)), cosphi; sin(0.25*atan2(y,x)), sinphi]      
      %pause
      R = [[cosphi, -sinphi]; [sinphi, cosphi]];
      P(:,[ii, jj]) = P(:,[ii, jj])*R; 
    end
  
    sum_v_old = sum_v;
    v = var(P.*P,1);  
    sum_v = sum(v);

    Msg = ['attempt: ' num2str(a) '; iter: ' num2str(b) '; total var: ' num2str(sum_v) '; max total var: ' num2str(sum_v_max)];
    if prprog
      prprogress(Options.verbose,['%s\n'],Msg);
    elseif Options.verbose
      disp(['varimax: ' Msg]);
    end  

    if (sum_v - sum_v_old) < Options.eps*sum_v_old
      ok = 1;   
      break;  
    end  
  end
  
  if ~ok
    Msg = ['the maximum number of iterations (' num2str(Options.max_iter) ') is reached before convergence'];
    if prprog
      prprogress(Options.verbose,['%s\n'],Msg);
    elseif Options.verbose
      disp(['varimax: ' Msg]);
    end  
  end 

  if sum_v > sum_v_max
    P_max = P;
    v_max = v;
    sum_v_max = sum_v;    
  end

  P = P_0;
end  

clear P_0 P;

if Options.sort
  [v_max, ind] = sort(v_max);
  v_max = fliplr(v_max);
  P_max = P_max(:,fliplr(ind));
end

return



