%VARIMAXFM Extracts sparse (pure) features 
%          
%  
%  U = VARIMAXFM
%  U = VARIMAXFM([], K_EXT, OPT)
%  U = VARIMAXFM([], FRAC, OPT)
%
%  W = A*U
%  [W, K_EXT, FRAC, V, P, Z, Y] = MAP(A,U)
%  [W, K_EXT, FRAC, V, P, Z, Y] = VARIMAXFM(A)
%  [W, K_EXT, FRAC, V, P, Z, Y] = VARIMAXFM(A, K_TO_EXT, OPT)
%  [W, K_EXT, FRAC, V, P, Z, Y] = VARIMAXFM(A, FRAC_TO_EXP, OPT)
%  Y = DATASET(W.DATA.ROT \ EYE(SIZE(W.DATA.ROT,1))) 
%
%  C = B*W
%  C = MAP(B, W)
%  C = VARIMAXFM(B, W)
%
% INPUT 
%   A             [M -by- K] training dataset
%   B             [N -by- K] test dataset
%
%   U             [K -by- ?] untrained VARIMAXFM mapping
%   W             [K -by- K_EXT] trained VARIMAXFM affine feature extraction mapping;
%   K_TO_EXT, 
%   FRAC_TO_EXT   
%                 the (integer) number of components to be extracted (1<= K_TO_EXT <= (K,M)) or 
%                 which fraction of variance to be explained (0 < FRAC_TO_EXP < 1) by PCA;
%                 NOTE: the number of PCs taken into account should be smaller than number of 
%                 objects/features in the training dataset (otherwise the result of algorithm is trivial);
%                 by default FRAC = 0.95
%
%   OPT.          optional VARIMAX settings 
%    N_ATTEMPTS   the number of independent attempts to maximize the varimax criterion; by default = 3 
%    MAX_ITER     the max number of iterations wihin each attempt; each iteration involves 
%                 optimization over all unsorted pairs visited in random order; by default = 3 
%    SORT         put the simplest factors first; by default = 1;
%    EPS          convergence criterion (NEW_VAR-OLD_VAR < EPS*OLD_VAR); by default = 1e-6
%    VERBOSE      force/disable messages; PRPROGRESS(VERBOSE,['%s\n'],MSG) will be run; 
%                 by default = []
%
% OUTPUT 
%   U            [K -by- ?] untrained VARIMAXOM mapping
%   W            [K -by- K_EXT] trained VARIMAXOM affine feature extraction mapping;
%   K_EXT        the actual number of extracted features; can be less than K_TO_EXT 
%   FRAC         [1 -by- K_EXT ] row of the varince fractions; FRAC(N) is variance fraction explained by first N PC's   
%                FRAC corresponds to the PCA, which is the first step of VARIMAX, not to the final feature extraction, 
%                except the FRAC(K_EXT) which is the total fraction of explained variance for both PCA and VARIMAX
%   V            [1 -by- K_EXT ] row of variances of squared factors (sparsity)
%   Z            [K -by- K_EXT] scores the result of mapping on training set
%   Y            [K_EXT -by- K] loadings (generalized inverse of mapping W: if Z = A*W, 
%                then A = Z*Y + error)            
%   C            [N -by- K_EXT] dataset, result of mapping execution on dataset B
%
%
%
% DESRIPTION
% This routine calculates what is called a raw Kaiser varimax. 
% Applied to the transposed data matrix.
%
% It is conceptiually close to the SIMPLISMA and OPA mappings. 
% It assumes that pure variables (concentration profiles) 
% have sparse structure. The output of the algorithm are extracted features  
% (linear combinations of original features). The letter F in
% the name of routine means that it is expected that exctracted pure
% features shows sparse behaviour for the training set objects.
% Loadings (pure objects or spectra) can be obtained either directly 
% or by Y = DATASET(W.DATA.ROT \ EYE(SIZE(W.DATA.ROT,1)))
%
% NOTE: PCA is done by low level routine not by PRTools PCA mapping.  
% No mean centering, no calss information is used. Scaling or
% any other type of preprocessing is the users responsibility.
%
% SEE ALSO
% OPAM, SIMPLISMAM, VARIMAXOM, SIMPLIMAPS, OPAPS, ALS, VARIMAX 

% Copyright: S.Verzakov, s.verzakov@ewi.tudelft.nl 
% Faculty EWI, Delft University of Technology
% P.O. Box 5031, 2600 GA Delft, The Netherlands

% $Id: varimaxfm.m,v 1.4 2007/01/31 21:52:19 serguei Exp $

function [w, k_ext, frac, v, z, y] = varimaxfm(a, w, Opt)

% No dataset given: return untrained mapping.
if (nargin < 1) | (isempty(a))
  if nargin < 2
    w = 0.95;
  end  
  if nargin < 3
    Opt = [];
  end  
  
  w = mapping(mfilename,'untrained',{w});
  w = setname(w,'VARIMAXFM Mapping');
  return
end

isdataset(a);           % Assert that A is a dataset 

% training
if nargin < 2 | ~isa(w,'mapping') 
  if nargin < 2 | isempty(w)
    k_to_ext_or_frac_to_expl = 0.95;  
  else  
    k_to_ext_or_frac_to_expl = w;  
  end  

  if nargin < 3
    Opt = [];  
  end  
  
  [m,k] = getsize(a);
  if k_to_ext_or_frac_to_expl >= m
    error('number of components to extract should be less than the number of objects in a training dataset');
  end  
  
  PCAOptions.M = 1;
  if k_to_ext_or_frac_to_expl >= 1
    PCAOptions.nPC = floor(k_to_ext_or_frac_to_expl);
  else
    PCAOptions.nPC    = nan;
    PCAOptions.nPC_em = 'PEV';
    PCAOptions.PEV    = k_to_ext_or_frac_to_expl;
  end
  
  [T, P, lambda, Options]  = pca_train((+a)',PCAOptions); 
  k_ext = length(lambda);
  frac  = cumsum(lambda/Options.Tr1);

  labin = a.featlab;
  featsize = a.featsize;
  a.data = zeros(m,1);

  [P_max,v] = varimax(P,Opt);
  
  R = P'*P_max;
  
  % normalization factor for loadings (spectra will be l2 normalized)
  h = sqrt(sum(repmat(lambda(:), [1 k_ext]).*(R.^2), 1));
  
  W = T * (repmat(1./lambda(:),[1 k_ext]).* R .* repmat(h,[k_ext 1]));
  
  
  if nargout >= 5
    % loadings
    %y = (W\eye(k));
    %W = (y'\eye(k))';
    
    y = dataset((repmat(1./h(:),[1 k_ext]).* R') *T');

    y.featlab  = labin;
    y.featsize = featsize;
    y.user     = a.user;
    y.name    = [y.name ': extracted spectra']; 
  end

  if nargout >= 6
    % scores
    % z = (+a)*W;
    
    z = a; clear a;
    z.data = P_max .* repmat(h,[m 1]);
    z.featlab = num2str([1:k_ext]','varivector%-1d');   
    z = remove_spectra_info(z);
    z.name    = [z.name ': extracted channels']; 
  end

  
  % Save all useful data
  w = affine(W,zeros(1,k_ext),labin,num2str([1:k_ext]','varivector%-1d'),size(W,1),size(W,2));
  w = setname(w,'VARIMAXFM Mapping');

% applying
else 
  % affine will do a job
  w = a*w;
  w = remove_spectra_info(w);
  w.name = [w.name ': exctracted channels'];
end

return

